/**@file

// libPackageCreationCommonFunctions.ctl
This library contains various functions used  in the package creation application.

@par Creation Date
  30/11/2011

@par Modification History
  30/11/2011 - First release
  
@par Constraints
  Can only be used by the main panel of the package creation application (dependencies to variables and widgets of the panel)

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@author 
  Cyril Caillaba (EN-ICE)
*/

//@{

/** Updates the tree view of the current mapping.
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL
*/
void updateMappingView()
{  
  treeCurrentMappingView.clear();
  for (int i = 1 ; i <= dynlen(mappingKeys(frontEndDeviceMapping)) ; i++)
  {
    // Note : the ID is prefixed by "FE" for a front-end and "DE_parentFrontEnd_" for a device, "W_parentFrontEnd_parentDevice" for a widget, to prevent errors with multiple items
    
    // Display the front-end
    treeCurrentMappingView.appendItem(
        "",
        "FE" + mappingKeys(frontEndDeviceMapping)[i],
        mappingKeys(frontEndDeviceMapping)[i]
    );
    for (int j = 1 ; j <= dynlen(frontEndDeviceMapping[mappingKeys(frontEndDeviceMapping)[i]]) ; j++)
    {
      // Display the device
      treeCurrentMappingView.appendItem(
          "FE" + mappingKeys(frontEndDeviceMapping)[i],
          "DE" + "_" + mappingKeys(frontEndDeviceMapping)[i] + "_" + frontEndDeviceMapping[mappingKeys(frontEndDeviceMapping)[i]][j],
          frontEndDeviceMapping[mappingKeys(frontEndDeviceMapping)[i]][j]
      );
      treeCurrentMappingView.ensureItemVisible("DE" + "_" + mappingKeys(frontEndDeviceMapping)[i] + "_" + frontEndDeviceMapping[mappingKeys(frontEndDeviceMapping)[i]][j]);
         
      for (int k = 1 ; k <= dynlen(deviceWidgetMapping[frontEndDeviceMapping[mappingKeys(frontEndDeviceMapping)[i]][j]]) ; k++)
      {
        // Display the widget   
        treeCurrentMappingView.appendItem(
            "DE" + "_" + mappingKeys(frontEndDeviceMapping)[i] + "_" + frontEndDeviceMapping[mappingKeys(frontEndDeviceMapping)[i]][j],
            "W" + "_" + mappingKeys(frontEndDeviceMapping)[i] + "_" + frontEndDeviceMapping[mappingKeys(frontEndDeviceMapping)[i]][j] + "_" + deviceWidgetMapping[frontEndDeviceMapping[mappingKeys(frontEndDeviceMapping)[i]][j]][k],
            deviceWidgetMapping[frontEndDeviceMapping[mappingKeys(frontEndDeviceMapping)[i]][j]][k]
        );
        treeCurrentMappingView.ensureItemVisible("W" + "_" + mappingKeys(frontEndDeviceMapping)[i] + "_" + frontEndDeviceMapping[mappingKeys(frontEndDeviceMapping)[i]][j] + "_" + deviceWidgetMapping[frontEndDeviceMapping[mappingKeys(frontEndDeviceMapping)[i]][j]][k]);
      
      
      }
    
    
    }
  }
}

/** Finds a given function in a file converted to a string (via fileToString())
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param stringFile input, a string representation of a file.
@param functionName input, the wanted function.
@return a string containing only the wanted function.
*/
string extractFunctionFromStringFile(string stringFile, string functionName)
{
  int functionPos = strpos(stringFile,"// " + functionName + "\n"); // the \n is needed in case a function name is equal to the beginning of another 
  string cutStringFile = substr(stringFile,functionPos); // Contains from the beginning of the function (including preceding comments) to the end
  int functionEndPos = strpos(cutStringFile,STRING_FUNCTION_SEPARATOR); // Finding the separator between 2 functions
  
  return substr(cutStringFile,0,functionEndPos);
}

//@}
