/**@file

// pcFGC2_PS_MPC.ctl
This library contains the widget, faceplate, etc. functions of FGC2_PS_MPC.

@par Creation Date
  04/05/2010

@par Modification History
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@author 
  H. Milcent (EN-ICE)
*/

//@{

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC2_PS_MPC_ObjectListGetValueTime
/** Function called from snapshot utility of the treeDeviceOverview to get the time and value
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDeviceName  input, device name
@param sDeviceType input, device type
@param dsReturnData output, return data, array of 5 strings
*/
pcFGC2_PS_MPC_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
float fB_MEAS;
int iSTATE_PC, iSTATE_OP, iST_FAULTS;
string sTime, sColor, sSTATE_PCLetter, sSTATE_OPLetter;
bool bBad, bStDevice;
mapping mSTATE_PC, mSTATE_OP;
string sSystemName, sFrontEnd, sB_MEAS;
bit32 b32FGC_MASK;
string sFE, sAppl, sBFormat, sBUnit;
int iBitPosFGC, iWF;
  
  pcFGC2_LHC_PC_loadTextLabel(PC_STATE_PC_TEXT_LABELDP, mSTATE_PC);
  pcFGC2_LHC_PC_loadTextLabel(PC_STATE_OP_TEXT_LABELDP, mSTATE_OP);

  sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);
  // get the PLC name 
  sFrontEnd = unGenericObject_GetPLCNameFromDpName(sDeviceName);
  sFrontEnd = sSystemName+sFrontEnd;

  dpGet(sDeviceName+".ProcessInput.B_MEAS", fB_MEAS, 
    sDeviceName+".ProcessInput.STATE_PC", iSTATE_PC,
    sDeviceName+".ProcessInput.STATE_PC:_online.._bad", bBad,
    sDeviceName+".ProcessInput.B_MEAS:_online.._stime", sTime,
    sDeviceName+".ProcessInput.ST_FAULTS", iST_FAULTS,
    sDeviceName+".ProcessInput.STATE_OP", iSTATE_OP,
    sDeviceName+".ProcessInput.ST_GTWDEV", bStDevice,
    sFrontEnd+ ".communication.FGC_MASK", b32FGC_MASK);

  pcFGC2_LHC_PC_getConfig(sDeviceName, sFE, sAppl, iBitPosFGC, iWF);
  sBFormat=dpGetFormat(sDeviceName + ".ProcessInput.B_MEAS");
  sBUnit=dpGetUnit(sDeviceName + ".ProcessInput.B_MEAS");

  sB_MEAS = strrtrim(unGenericObject_FormatValue(sBFormat, fB_MEAS) + " " + sBUnit);
  
  if(!getBit(b32FGC_MASK, iBitPosFGC))
    { // FGC offline
    iSTATE_PC = PC_TEXT_LABEL_FGC_OFFLINE;
    iSTATE_OP = PC_TEXT_LABEL_FGC_OFFLINE;
    //sI_MEAS = PC_FGC_UNKNOWN_VALUE;
    bBad=TRUE;
    }
  else if(!bStDevice)
    {
    //iSTATE_PC = PC_TEXT_LABEL_FGC_UNKNOWN;
    //iSTATE_OP = PC_TEXT_LABEL_FGC_UNKNOWN;
    //sI_MEAS = PC_FGC_UNKNOWN_VALUE;
    bBad=TRUE;
    }
  
  dsReturnData[1] = sTime;
  dsReturnData[2] = "B="+sB_MEAS;
  dsReturnData[3] = bBad;
  
  if(mappingHasKey(mSTATE_PC, iSTATE_PC))
    {
    sSTATE_PCLetter = mSTATE_PC[iSTATE_PC][PC_TEXT_LABEL_TEXT];
    }
  else
    {
    sSTATE_PCLetter = PC_FGC_UNKNOWN_VALUE;
    }
  
  if(mappingHasKey(mSTATE_OP, iSTATE_OP))
    {
    sSTATE_OPLetter = mSTATE_OP[iSTATE_OP][PC_TEXT_LABEL_TEXT];
    }
  else
    {
    sSTATE_OPLetter = PC_FGC_UNKNOWN_VALUE;
    }
    
  if(!bStDevice && (getBit(b32FGC_MASK, iBitPosFGC)))
    dsReturnData[4] = "NO FGC2_PS_MPC CMW DEVICE";
  else
    dsReturnData[4] = sSTATE_PCLetter+"      " +sSTATE_OPLetter;
  
  if(iST_FAULTS != 0) 
    sColor = "unFaceplate_AlarmActive";
  else
    sColor = "unSynoptic_staticText";
  
  dsReturnData[5] = sColor;
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "FGC snapshot FGC2_PS_MPC "+sDeviceName, "pcFGC2_PS_MPC_ObjectListGetValueTime", sDeviceName, dsReturnData);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC2_PS_MPC_WidgetRegisterCB
/** widget DistributedControl callback
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDp input, the DistributedControl system name DP name
@param bSystemConnected input, the state of the system name
*/
pcFGC2_PS_MPC_WidgetRegisterCB(string sDp, bool bSystemConnected)
{
  string deviceName, sSystemName;
  int iAction;
  dyn_string exceptionInfo;
  string sFrontEnd, sSystFE;
  bool bRemote;
  
  deviceName = unGenericDpFunctions_getWidgetDpName($sIdentifier);
  if (deviceName == "")  // In case of disconnection
  {
    deviceName = g_sDpName;
  }
  g_bUnSystemAlarmPlc = false;
// get the systemName
  sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
  sFrontEnd = unGenericObject_GetPLCNameFromDpName(deviceName);

  unDistributedControl_isRemote(bRemote, sSystemName);
  if(bRemote)
    g_bSystemConnected = bSystemConnected;
  else
    g_bSystemConnected = true;
    
  if(sFrontEnd != "")
    {
    sSystFE = sSystemName+c_unSystemAlarm_dpPattern+DS_pattern+sFrontEnd;
    sFrontEnd = sSystemName+sFrontEnd;
//DebugN(sSystemName, sFrontEnd);
    if(dpExists(sSystFE))
      g_bUnSystemAlarmPlc = true;
    }

  unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "FGC widget FGC2_PS_MPC "+deviceName, "pcFGC2_PS_MPC_WidgetRegisterCB", sDp, deviceName, g_bCallbackConnected, iAction);
  switch (iAction)
    {
    case UN_ACTION_DISCONNECT:
      pcFGC2_PS_MPC_WidgetDisconnection(g_sWidgetType);
      break;
    case UN_ACTION_DPCONNECT:
      g_sDpName = deviceName;
      pcFGC2_PS_MPC_WidgetConnect(deviceName, sSystFE, sFrontEnd);
      break;
    case UN_ACTION_DPDISCONNECT_DISCONNECT:
      pcFGC2_PS_MPC_WidgetDisconnect(deviceName, sSystFE, sFrontEnd);
      pcFGC2_PS_MPC_WidgetDisconnection(g_sWidgetType);
      break;
    case UN_ACTION_NOTHING:
    default:
      break;
    }
}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC2_PS_MPC_WidgetDisconnection
/** set the widget when the system is disconnected 
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sWidgetType input, the type of widget
*/
pcFGC2_PS_MPC_WidgetDisconnection(string sWidgetType)
{
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "FGC widget FGC2_PS_MPC", "pcFGC2_PS_MPC_WidgetDisconnection", sWidgetType);
    switch(sWidgetType)
    {
      case "FGC2_PS_MPC":
        setMultiValue("STATE_PC", "color", "unDataNoAccess",
                      "STATE_OP", "color", "unDataNoAccess",
                      "WarningText", "text", "",
                      "AlarmText", "text", "",
                      "WidgetArea", "visible", false,
                      "SelectArea", "foreCol", "",
                      "LockBmp", "visible", false,
                      "B_MEAS", "foreCol", "unDataNoAccess",
                      "V_MEAS", "foreCol", "unDataNoAccess",
                      "BorderAlarm", "foreCol", "pcWidgetBorderOKColor");
        break;
      case "DIVPAM_ID":
        setMultiValue("DIVPAM", "text", "xxxxx", "DIVPAM", "foreCol", "unDataNoAccess", "WarningText", "text", "");
        break;
    }
}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC2_PS_MPC_WidgetConnect
/** dpConnect to the device data
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, front-end device DP name
@param sSystFE input, system integrity name
@param sFrontEnd input, front-end device name
*/
pcFGC2_PS_MPC_WidgetConnect(string deviceName, string sSystFE, string sFrontEnd)
{
  int iRes, iWF;
  string sSystemName;
  string sFE, sAppl;
  
//  check if the device name is of type "pcFGC2_PS_MPC" if not like of no PLC. -> not available.
  if(dpTypeName(deviceName) != "FGC2_PS_MPC")
    g_bUnSystemAlarmPlc = false;
  
  sSystemName = unGenericDpFunctions_getSystemName(deviceName);
  
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "FGC widget FGC2_PS_MPC "+deviceName, "pcFGC2_PS_MPC_WidgetConnect", deviceName, g_bUnSystemAlarmPlc);
  if(g_bUnSystemAlarmPlc)
  {
    switch(g_sWidgetType)
    {
      case "FGC2_PS_MPC":
        pcFGC2_LHC_PC_getConfig(deviceName, sFE, sAppl, g_iBitPosFGC, iWF);
        pcFGC2_LHC_PC_loadTextLabel(PC_STATE_PC_TEXT_LABELDP, g_mSTATE_PC);
        pcFGC2_LHC_PC_loadTextLabel(PC_STATE_OP_TEXT_LABELDP, g_mSTATE_OP);
        g_sBFormat=dpGetFormat(deviceName + ".ProcessInput.B_MEAS");
        g_sBUnit=dpGetUnit(deviceName + ".ProcessInput.B_MEAS");    

        g_sVFormat=dpGetFormat(deviceName + ".ProcessInput.V_MEAS");
        g_sVUnit=dpGetUnit(deviceName + ".ProcessInput.V_MEAS");

        iRes = dpConnect("pcFGC2_PS_MPC_WidgetCB", 
                           deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                           deviceName + ".statusInformation.selectedManager",
                           deviceName + ".ProcessInput.STATE_PC:_online.._invalid",
                           deviceName + ".ProcessInput.ST_FAULTS",
                           deviceName + ".ProcessInput.ST_WARNINGS",
                           deviceName + ".ProcessInput.STATE_PC",
                           deviceName + ".ProcessInput.STATE_OP",
                           deviceName + ".ProcessInput.B_MEAS",
                           deviceName + ".ProcessInput.V_MEAS",
                           deviceName + ".ProcessInput.ST_GTWDEV",
                           sFrontEnd + ".communication.FGC_MASK",
                           sSystFE+".alarm",
                           sSystFE+".enabled");
        g_bCallbackConnected = (iRes >= 0);    
        break;
      case "DIVPAM_ID":
        setValue("DIVPAM", "foreCol", "black");
        iRes = dpConnect("pcFGC2_PS_MPC_DIVPAMWidgetCB", 
                           deviceName + ".ProcessInput.STATE_PC:_online.._invalid",
                           deviceName + ".ProcessInput.PAL_CHAN_DIVPAM_ID",
                           deviceName + ".ProcessInput.ST_GTWDEV",
                           sFrontEnd + ".communication.FGC_MASK",
                           sSystFE+".alarm",
                           sSystFE+".enabled");
        g_bCallbackConnected = (iRes >= 0);    
        break;
    }
  }
  else
    pcFGC2_PS_MPC_WidgetDisconnection(g_sWidgetType);

}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC2_PS_MPC_WidgetDisconnect
/** dpDisconnect to the device data
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, front-end device DP name
@param sSystFE input, system integrity name
@param sFrontEnd input, front-end device name
*/
pcFGC2_PS_MPC_WidgetDisconnect(string deviceName, string sSystFE, string sFrontEnd)
{
  int iRes;

  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "FGC widget FGC2_PS_MPC "+deviceName, "pcFGC2_PS_MPC_WidgetDisconnect", deviceName, g_bUnSystemAlarmPlc);
  if(g_bUnSystemAlarmPlc)
  {
    switch(g_sWidgetType)
    {
      case "FGC2_PS_MPC":
        iRes = dpDisconnect("pcFGC2_PS_MPC_WidgetCB", 
                               deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                               deviceName + ".statusInformation.selectedManager",
                               deviceName + ".ProcessInput.STATE_PC:_online.._invalid",
                               deviceName + ".ProcessInput.ST_FAULTS",
                               deviceName + ".ProcessInput.ST_WARNINGS",
                               deviceName + ".ProcessInput.STATE_PC",
                               deviceName + ".ProcessInput.STATE_OP",
                               deviceName + ".ProcessInput.B_MEAS",
                               deviceName + ".ProcessInput.V_MEAS",
                               deviceName + ".ProcessInput.ST_GTWDEV",
                               sFrontEnd + ".communication.FGC_MASK",
                               sSystFE+".alarm",
                               sSystFE+".enabled"
                            );
        g_bCallbackConnected = !(iRes >= 0);
        break;
      case "DIVPAM_ID":
        iRes = dpDisconnect("pcFGC2_PS_MPC_DIVPAMWidgetCB", 
                           deviceName + ".ProcessInput.STATE_PC:_online.._invalid",
                           deviceName + ".ProcessInput.PAL_CHAN_DIVPAM_ID",
                           deviceName + ".ProcessInput.ST_GTWDEV",
                           sFrontEnd + ".communication.FGC_MASK",
                           sSystFE+".alarm",
                           sSystFE+".enabled");
        g_bCallbackConnected = !(iRes >= 0);
        break;
    }
  }
}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC2_PS_MPC_WidgetCB
/** callback function on the device data
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDpLocked input, the lock DPE 
@param bLocked input, the lock state 
@param sDpSelectedManager input, the selected manager DPE 
@param sSelectedManager input, the selected manager 
@param sDpInvalid input, the invalid DPE 
@param bInvalid input, data validity
@param sDpST_FAULTS input, the ST_FAULTS DPE 
@param iST_FAULTS input, the ST_FAULTS value
@param sDpST_WARNINGS input, the ST_WARNINGS DPE 
@param iST_WARNINGS input, the ST_WARNINGS value
@param sDpSTATE_PC input, the STATE_PC DPE 
@param iSTATE_PC input, the STATE_PC value
@param sDpSTATE_OP input, the STATE_OP DPE 
@param iSTATE_OP input, the STATE_OP value
@param sDpB_MEAS input, the B_MEAS DPE 
@param fB_MEAS input, the B_MEAS value
@param sDpV_MEAS input, the V_MEAS DPE 
@param fV_MEAS input, the V_MEAS value
@param sDpDevice input, the ST_GTWDEV DPE 
@param bStDevice input, the ST_GTWDEV value
@param sDpFGC_MASK input, the GTW FGC_MASK DPE 
@param b32FGC_MASK input, the GTW FGC_MASK value
@param sDpFESystemIntegrityAlarmValue input, front-end device system integrity alarm DPE 
@param iFESystemIntegrityAlarmValue input, front-end device system integrity alarm value
@param sDpFESystemIntegrityAlarmEnabled input, front-end device system integrity enable DPE
@param bFESystemIntegrityAlarmEnabled input, front-end device system integrity enable value
*/
pcFGC2_PS_MPC_WidgetCB(string sDpLocked, bool bLocked,
                    string sDpSelectedManager, string sSelectedManager,
                    string sDpInvalid, bool bInvalid,
                    string sDpST_FAULTS, int iST_FAULTS,
                    string sDpST_WARNINGS, int iST_WARNINGS,
                    string sDpSTATE_PC, int iSTATE_PC,
                    string sDpSTATE_OP, int iSTATE_OP,
                    string sDpB_MEAS, float fB_MEAS,
                    string sDpV_MEAS, float fV_MEAS,
                    string sDpDevice, bool bStDevice,
                    string sDpFGC_MASK, bit32 b32FGC_MASK,
                    string sDpSystemIntegrityAlarmValue, int iSystemIntegrityAlarmValue, 
                    string sDpSystemIntegrityAlarmEnabled, bool bSystemIntegrityAlarmEnabled)
{
  string selectColor, sAlarmColor, sWarningLetter, sAlarmLetter, sWarningColor, sSTATE_PCColor, sSTATE_OPColor, sSTATE_PCLetter, sSTATE_OPLetter;
  bool bSelectVisible, bInvalidData;;
  string sB_MEAS, sMEASColor, sV_MEAS;
  string sBodyColor;
  
  if(g_bSystemConnected)
  {
    unGenericObject_WidgetSelectAnimation(bLocked, sSelectedManager, selectColor, bSelectVisible);
    
    sB_MEAS = strrtrim(unGenericObject_FormatValue(g_sBFormat, fB_MEAS) + " " + g_sBUnit);
    sV_MEAS = strrtrim(unGenericObject_FormatValue(g_sVFormat, fV_MEAS) + " " + g_sVUnit);
    
    if(iST_WARNINGS != 0)
    {
      sAlarmLetter = "W";
      sAlarmColor = "unFaceplate_AlarmActive";
    }
      
    if(iST_FAULTS != 0)
      sBodyColor = "unFaceplate_AlarmActive";
    else
      sBodyColor = "pcWidgetBorderOKColor";
      
    if(!getBit(b32FGC_MASK, g_iBitPosFGC))
    {
      // set state OP to unknown and state PC to OFFLINE
      iSTATE_PC = PC_TEXT_LABEL_FGC_OFFLINE;
      iSTATE_OP = PC_TEXT_LABEL_FGC_OFFLINE;
      sWarningLetter = UN_WIDGET_TEXT_OLD_DATA;
      sAlarmLetter = "";
      sAlarmColor = "";      
      bInvalidData = TRUE;  
    }
    else if(!bStDevice)
    {
      sWarningLetter = UN_WIDGET_TEXT_OLD_DATA;
      sAlarmLetter = "";
      sAlarmColor = "";      
      bInvalidData = TRUE;      
    }  
    else if (bInvalid)
    {
      sWarningLetter = UN_WIDGET_TEXT_INVALID;
      bInvalidData = TRUE;
    }  
    else if (!bSystemIntegrityAlarmEnabled || (iSystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value))
    {
      bInvalidData = TRUE;
      sWarningLetter = UN_WIDGET_TEXT_OLD_DATA;
    }
    
    if (bInvalidData)
    {
      sWarningColor = "unDataNotValid";      
      sMEASColor = "unDataNotValid";      
    }
    else
    {
      //sMEASColor = "unWidget_ControlStateAuto";  
      sMEASColor = "unDisplayValue_Status";      
      sWarningLetter = "";
    }    
    
    // get STATE_PC, STATE_OP
    if(mappingHasKey(g_mSTATE_PC, iSTATE_PC))
    {
      sSTATE_PCLetter = g_mSTATE_PC[iSTATE_PC][PC_TEXT_LABEL_TEXT];
      if (bInvalidData) 
        sSTATE_PCColor = "unDataNotValid";
      else 
        sSTATE_PCColor = g_mSTATE_PC[iSTATE_PC][PC_TEXT_LABEL_COLOR];
    }
    else
    {
      sSTATE_PCLetter = PC_FGC_UNKNOWN_VALUE;
      sSTATE_PCColor = "unDataNotUpToDate";
    }
    
    if(mappingHasKey(g_mSTATE_OP, iSTATE_OP))
    {
      sSTATE_OPLetter = g_mSTATE_OP[iSTATE_OP][PC_TEXT_LABEL_TEXT];
      if (bInvalidData) 
        sSTATE_OPColor = "unDataNotValid";    
      else 
        sSTATE_OPColor = g_mSTATE_OP[iSTATE_OP][PC_TEXT_LABEL_COLOR];
    }
    else
    {
      sSTATE_OPLetter = PC_FGC_UNKNOWN_VALUE;
      sSTATE_OPColor = "unDataNotUpToDate";
    }

    unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "FGC widget FGC2_PS_MPC "+sDpLocked, "pcFGC2_PS_MPC_WidgetCB", sDpLocked, g_sWidgetType, 
                                 sWarningLetter, sWarningColor, g_bSystemConnected,
                                 selectColor, bSelectVisible);
    setMultiValue("WarningText", "text", sWarningLetter, 
                    "WarningText", "foreCol", sWarningColor,
                    "AlarmText", "text", sAlarmLetter, 
                    "AlarmText", "foreCol", sAlarmColor,
                    "STATE_PC", "text", sSTATE_PCLetter, 
                    "STATE_PC", "foreCol", sSTATE_PCColor,
                    "STATE_OP", "text", sSTATE_OPLetter, 
                    "STATE_OP", "foreCol", sSTATE_OPColor,
                    "SelectArea", "foreCol", selectColor,
                    "WidgetArea", "visible", true,
                    "LockBmp", "visible", bSelectVisible,
                    "B_MEAS", "text", sB_MEAS,
                    "B_MEAS", "foreCol", sMEASColor,
                    "V_MEAS", "text", sV_MEAS,
                    "V_MEAS", "foreCol", sMEASColor,
                    "BorderAlarm", "foreCol", sBodyColor);
  }
}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC2_PS_MPC_DIVPAMWidgetCB
/** callback function on the device data
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDpInvalid input, the invalid DPE 
@param bInvalid input, data validity
@param sDp input, the DIVPAM_ID DPE 
@param diDIVPAM input, the DIVPAM_ID value
@param sDpDevice input, the ST_GTWDEV DPE 
@param bStDevice input, the ST_GTWDEV value
@param sDpFGC_MASK input, the GTW FGC_MASK DPE 
@param b32FGC_MASK input, the GTW FGC_MASK value
@param sDpFESystemIntegrityAlarmValue input, front-end device system integrity alarm DPE 
@param iFESystemIntegrityAlarmValue input, front-end device system integrity alarm value
@param sDpFESystemIntegrityAlarmEnabled input, front-end device system integrity enable DPE
@param bFESystemIntegrityAlarmEnabled input, front-end device system integrity enable value
*/
pcFGC2_PS_MPC_DIVPAMWidgetCB(string sDpInvalid, bool bInvalid,
                            string sDp, dyn_int diDIVPAM,
                            string sDpDevice, bool bStDevice,
                            string sDpFGC_MASK, bit32 b32FGC_MASK,
                            string sDpSystemIntegrityAlarmValue, int iSystemIntegrityAlarmValue, 
                            string sDpSystemIntegrityAlarmEnabled, bool bSystemIntegrityAlarmEnabled)
{
  string sWarningLetter, sWarningColor;
  string sText;
  
  if(g_bSystemConnected)
  {
    if(!getBit(b32FGC_MASK, g_iBitPosFGC))
    {
      sWarningLetter = UN_WIDGET_TEXT_OLD_DATA;
      sWarningColor = "unDataNotValid";
    }
    else if(!bStDevice)
    {
      sWarningLetter = UN_WIDGET_TEXT_OLD_DATA;
      sWarningColor = "unDataNotValid";
    }  
    else if (bInvalid)
    {
      sWarningLetter = UN_WIDGET_TEXT_INVALID;
      sWarningColor = "unDataNotValid";
    }  
    else if (!bSystemIntegrityAlarmEnabled || (iSystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value))
    {
      sWarningLetter = UN_WIDGET_TEXT_OLD_DATA;
      sWarningColor = "unDataNotValid";
    }
    unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "FGC widget DIVPAM "+sDp, "pcFGC2_PS_MPC_DIVPAMWidgetCB", g_iDIVPAM_position, 
                                 sWarningLetter, sWarningColor, g_bSystemConnected);

    if(g_iDIVPAM_position<=dynlen(diDIVPAM))
      sText = diDIVPAM[g_iDIVPAM_position];
    else
      sText = "????";
    setMultiValue("WarningText", "text", sWarningLetter, 
                    "WarningText", "foreCol", sWarningColor,
                    "DIVPAM", "text", sText);
  }
}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC2_PS_MPC_MenuConfiguration
/** pop-up menu
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDpName input, device DP name
@param sDpType input, device type
@param dsAccessOk input, the access control 
@param menuList output, pop-up menu to show, dyn_string to be given to the popupMenu function
*/
pcFGC2_PS_MPC_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
{
  dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_SELECT_TEXT, UN_POPUPMENU_FACEPLATE_TEXT, UN_POPUPMENU_TREND_TEXT);
  bool bAddress=pcFGC2_PS_MPC_checkAddresOutput(sDpName);
  dyn_string dsUserAccess;
  int pos,i, len, iMenu=UN_POPUPMENU_MIN;
  string sMenu, sAccess, sMenuText;
  bool bSelected;
  bool bActionOk;
  dyn_string dsButtons = makeDynString(PC_FACEPLATE_BUTTON_PAL_CHAN_DIVPAM_ID,
                                        PC_FACEPLATE_BUTTON_CONFIG_MODE); 
  
  if(bAddress) 
    unGenericObject_addSelectToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
  unGenericObject_addUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
  unGenericObject_addDefaultUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
  unGenericObject_addTrendActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
  if(bAddress) {
    // add here the device actions
    pos = dynContains(menuList, "PUSH_BUTTON, Device configuration, 10, 1");
    if(pos > 0) {
      // get the allowed device actions for the user logged in
      pcFGC2_PS_MPC_ButtonUserAccess(sDpName, "FGC2_PS_MPC", dsUserAccess);
      bSelected = dynContains(menuList, "PUSH_BUTTON, Deselect, 1, 1");
      // remove Select if it is in the list
      len = dynContains(dsUserAccess, UN_POPUPMENU_SELECT_TEXT);
      if(len > 0)
        dynRemove(dsUserAccess, len);
      len = dynlen(dsButtons);
      if(dynlen(dsUserAccess) > 0)
        dynInsertAt(menuList, "SEPARATOR", pos);

      // add the device actions
      for(i=1;i<=len;i++)
      {
        sprintf(sMenu, "%d", (iMenu+1));
        iMenu++;
        sAccess = "0";
        bActionOk = (dynContains(dsUserAccess, dsButtons[i]) > 0);  // User access
        switch(dsButtons[i])
        {
          case PC_FACEPLATE_BUTTON_PAL_CHAN_DIVPAM_ID:
            bActionOk = bActionOk & bSelected ;
            sMenuText = PC_SET_VALUE_PAL_CHAN_DIVPAM_ID;
            break;
          case PC_FACEPLATE_BUTTON_CONFIG_MODE:
            bActionOk = bActionOk & bSelected ;
            sMenuText = PC_SET_VALUE_CONFIG_MODE;
            break;
          default:
            bActionOk = false;
            sMenuText = "";
            break;
        }
        if(bActionOk)
          sAccess = "1";
        if(sMenuText != "")
          dynInsertAt(menuList, "PUSH_BUTTON, "+sMenuText+", "+sMenu+", "+sAccess, pos);
      }
      if(dynlen(dsUserAccess) > 0)
        dynInsertAt(menuList, "SEPARATOR", pos);
    }
  }
  
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "FGC rigthClick FGC2_PS_MPC "+sDpName, "pcFGC2_PS_MPC_MenuConfiguration", sDpName, sDpType, dsAccessOk, dsUserAccess, menuList);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC2_PS_MPC_checkAddresOutput
/** check if the output DPE have an address config
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDpName input, device DP name
@return true if address config exists/false if not
*/
bool pcFGC2_PS_MPC_checkAddresOutput(string sDpName)
{
  bool bExist, bActive;
  dyn_anytype daConfig;
  dyn_string exInfo;
  
  fwPeriphAddress_get(sDpName+".ProcessOutput.PAL_CHAN_DIVPAM_ID", bExist, daConfig, bActive, exInfo);
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "FGC FGC2_PS_MPC "+sDpName, "pcFGC2_PS_MPC_checkAddresOutput", sDpName, bExist, daConfig, bActive, exInfo);
  return bExist;
}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC2_PS_MPC_HandleMenu
/** handle the answer of the popup menu
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, device DP name
@param sDpType input, device type
@param menuList input, the access control 
@param iMenuAnswer input, selected menu value 
*/
pcFGC2_PS_MPC_HandleMenu(string deviceName, string sDpType, dyn_string menuList, int iMenuAnswer)
{
  int iUN_POPUPMENU_MIN = UN_POPUPMENU_MIN;
  dyn_string exceptionInfo;
  dyn_string dsTemp, dsSplit;
  int len, i, iMenu;
  
  // handle unicos menu
  if(iMenuAnswer<iUN_POPUPMENU_MIN)
    unGenericObject_HandleUnicosMenu(deviceName, sDpType, menuList, iMenuAnswer, exceptionInfo);
  else { // handle the device actions
    // find the corresponding device action text
    dsTemp = dynPatternMatch("PUSH_BUTTON, *, 1", menuList);
    len = dynlen(dsTemp);
    for(i=1;i<=len;i++)
    {
      dsSplit = strsplit(dsTemp[i], ",");
      sscanf(dsSplit[3], "%d", iMenu);
      if(iMenu == iMenuAnswer) {
        switch(strltrim(dsSplit[2], " ")) {
          case PC_SET_VALUE_PAL_CHAN_DIVPAM_ID:
            pcFGC2_PS_MPC_OpenSetPanel(PC_SET_DIVPAM_PANEL, PC_SET_VALUE_PAL_CHAN_DIVPAM_ID, deviceName);
            break;
          case PC_SET_VALUE_CONFIG_MODE:
            pcFGC2_PS_MPC_OpenSetPanel(PC_SET_CONFIGMODE_PANEL, PC_SET_VALUE_CONFIG_MODE, deviceName);
            break;
        }
      }
    }
  }
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "FGC rigthClick FGC2_PS_MPC "+deviceName, "pcFGC2_PS_MPC_HandleMenu", deviceName, sDpType, menuList, iMenuAnswer);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC2_PS_MPC_ButtonUserAccess
/** returns the list of allowed action on the device for a user logged in
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDpName input, the device DP name
@param sDpType input, the device type
@param dsAccess input, list of allowed action on the device
*/
pcFGC2_PS_MPC_ButtonUserAccess(string sDpName, string sDpType, dyn_string &dsAccess)
{
  bool operator, expert, admin;
  int i, length;
  dyn_bool dbPermissions;
  dyn_string exceptionInfo;
  string sOperAction, sExpAction, sAdminAction;
  bool bActionDefined = false;

  unGenericDpFunctions_getAccessControlPriviledgeRigth(sDpName, g_dsDomainAccessControl, dbPermissions, sOperAction, sExpAction, sAdminAction, exceptionInfo);
  if((sOperAction != UNICOS_ACCESSCONTROL_DOMAINNAME) || (sExpAction != UNICOS_ACCESSCONTROL_DOMAINNAME) ||
     (sAdminAction != UNICOS_ACCESSCONTROL_DOMAINNAME))
    bActionDefined=true;
  operator = dbPermissions[2];
  expert = dbPermissions[3];
  admin = dbPermissions[4];
  dsAccess = makeDynString();
  if (operator)
  {
    if(bActionDefined)
      dynAppend(dsAccess, strsplit(sOperAction, UN_ACCESS_CONTROL_SEPARATOR));
  }
  if (expert)
  {
    if(bActionDefined)
      dynAppend(dsAccess, strsplit(sExpAction, UN_ACCESS_CONTROL_SEPARATOR));
  }          
  if(admin)
  {
    if(bActionDefined)
      dynAppend(dsAccess, strsplit(sAdminAction, UN_ACCESS_CONTROL_SEPARATOR));
    else
      dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_SELECT, 
                                        PC_FACEPLATE_BUTTON_PAL_CHAN_DIVPAM_ID,
                                        PC_FACEPLATE_BUTTON_CONFIG_MODE));
  }
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "FGC button FGC2_PS_MPC "+sDpName, "pcFGC2_PS_MPC_ButtonUserAccess", sDpName, sDpType, dbPermissions, sOperAction, sExpAction, sAdminAction, dsAccess);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC2_PS_MPC_ButtonRegisterCB
/** Contextual button DistributedControl callback of the contextual device button panel
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDp input, the DistributedControl system name DP name
@param bSystemConnected input, the state of the system name
*/
pcFGC2_PS_MPC_ButtonRegisterCB(string sDpSystemConnected, bool bSystemConnected)
{
  string deviceName, sSystemName, sDeviceType, sFunction;
  int iAction, iRes, iManReg01;
  dyn_string dsFunctions, exceptionInfo, dsUserAccess;
  
  deviceName = unGenericDpFunctions_getDpName($sDpName);
  sSystemName = unGenericDpFunctions_getSystemName(deviceName);
  
  unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "FGC button FGC2_PS_MPC "+deviceName, "pcFGC2_PS_MPC_ButtonRegisterCB", deviceName, g_bCallbackConnected, iAction);
  switch (iAction)
    {
    case UN_ACTION_DISCONNECT:
      pcFGC2_PS_MPC_ButtonDisconnection();
      break;
    case UN_ACTION_DPCONNECT:
// call the DeviceType_ButtonUserAccess to get the list of allowed buttons.

      sDeviceType=dpTypeName(deviceName);
      unGenericObject_GetFunctions(sDeviceType, dsFunctions, exceptionInfo);
      sFunction = dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS];
             
      if (sFunction != "")
      {
        evalScript(dsUserAccess, "dyn_string main(string deviceNameTemp, string sDpTypeTemp) {" + 
               "dyn_string dsUserAccessReturn;" +
               "if (isFunctionDefined(\"" + sFunction  + "\"))" +
               "    {" +
                 sFunction  + "(deviceNameTemp, sDpTypeTemp, dsUserAccessReturn);" +
               "    }" +
               "return dsUserAccessReturn; }", makeDynString(), deviceName, sDeviceType);
            
        g_dsUserAccess=dsUserAccess;  
      }
      else
        fwException_raise(exceptionInfo,"ERROR", "pcFGC2_PS_MPC_ButtonRegisterCB: " + getCatStr("unGeneration","UNKNOWNFUNCTION"), "EMPTY");

      g_bAddress=pcFGC2_PS_MPC_checkAddresOutput($sDpName);
      unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "FGC button FGC2_PS_MPC "+deviceName, "pcFGC2_PS_MPC_ButtonRegisterCB Connect", deviceName, sDeviceType, dsUserAccess, g_bAddress);
      iRes = dpConnect("pcFGC2_PS_MPC_ButtonAnimationCB",
                            deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                            deviceName + ".statusInformation.selectedManager");
      g_bCallbackConnected = (iRes >= 0);
      break;
    case UN_ACTION_DPDISCONNECT_DISCONNECT:
      unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "FGC button FGC2_PS_MPC "+deviceName, "pcFGC2_PS_MPC_ButtonRegisterCB Disconnect", deviceName);
      iRes = dpDisconnect("pcFGC2_PS_MPC_ButtonAnimationCB",
                            deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                            deviceName + ".statusInformation.selectedManager");
        g_bCallbackConnected = !(iRes >= 0);
      pcFGC2_PS_MPC_ButtonDisconnection();
      break;
    case UN_ACTION_NOTHING:
    default:
      break;
    }
}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC2_PS_MPC_ButtonAnimationCB
/** callback function on the device data
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDpLock input, the lock DPE 
@param bLocked input, selected lock state of the device
@param sDpSelect input, the select DPE 
@param sSelectedManager input, user and Ui that selected the device
*/
pcFGC2_PS_MPC_ButtonAnimationCB(string sDpLock, bool bLocked, string sDpSelect, string sSelectedManager)
{
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "FGC button FGC2_PS_MPC "+sDpLock, "pcFGC2_PS_MPC_ButtonAnimationCB", sDpLock, g_dsUserAccess);
  pcFGC2_PS_MPC_ButtonSetState(unGenericDpFunctions_getDpName(sDpLock), "", g_dsUserAccess, 
                                makeDynString(bLocked, sSelectedManager, g_bAddress, "", "", "", ""));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC2_PS_MPC_ButtonSetState
/** Set the state of the contextual button of the device
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device DP name
@param sDpType input, the device type
@param dsUserAccess input, list of allowed action on the device
@param dsData input, the device data [1] = lock state, [2] = lock by, [3] .. [6] device data
*/
pcFGC2_PS_MPC_ButtonSetState(string deviceName, string sDpType, dyn_string dsUserAccess, dyn_string dsData)
{
  bool bSelected, bAddress;
  string localManager;
  bool bLocked;
  string sSelectedManager;
  
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "FGC button FGC2_PS_MPC "+deviceName, "pcFGC2_PS_MPC_ButtonSetState", deviceName, sDpType, dsUserAccess, dsData);
  bLocked = dsData[1];
  sSelectedManager = dsData[2];
  bAddress = dsData[3];
  
// 1. Selection state
  localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedManager);
  bSelected = (localManager == "S");

//DebugN(myModuleName(), myPanelName());
// check if contextuel button in unicosHMI: yes --> do not show the buttons
  if(myPanelName() == "contextualButton.pnl;"+deviceName)
    bAddress = false;
   //3. Button Select state
  if(bAddress) {
// set all buttons state and set text
    setMultiValue("buttonPAL_CHAN_DIVPAM_ID", "enabled", bSelected & (dynContains(dsUserAccess, PC_FACEPLATE_BUTTON_PAL_CHAN_DIVPAM_ID) > 0), 
                  "buttonCONFIG_MODE", "enabled", bSelected & (dynContains(dsUserAccess, PC_FACEPLATE_BUTTON_CONFIG_MODE) > 0),
                  "buttonPAL_CHAN_DIVPAM_ID", "text", PC_SET_VALUE_PAL_CHAN_DIVPAM_ID,
                  "buttonCONFIG_MODE", "text", PC_SET_VALUE_CONFIG_MODE);
    unGenericObject_ButtonAnimateSelect(localManager, (dynContains(dsUserAccess, UN_FACEPLATE_BUTTON_SELECT) > 0));  
//DebugN(bSelected, iSTATE_PC, dsUserAccess, dbState, bOffWorking, bOnWorking);
  }
  else {
// disable all buttons, set text to ""
    setMultiValue("buttonPAL_CHAN_DIVPAM_ID", "enabled", false, "buttonPAL_CHAN_DIVPAM_ID", "text", "",
                  "buttonCONFIG_MODE", "enabled", false, "buttonCONFIG_MODE", "text", "",
                  "buttonSelect", "text", "","buttonSelect", "enabled", false);
  }
}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC2_PS_MPC_UserLoginGetButtonState
/** return the device data, this function is called when the user logs in and logs out
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device DP name
@param sType input, the device type
@param dsReturnData output, the device data, [1] = lock state, [2] = lock by, [3] .. [6] device data
*/
pcFGC2_PS_MPC_UserLoginGetButtonState(string deviceName, string sType, dyn_string &dsReturnData)
{
  int iRes, iSTATE_PC, iST_UNLATCHED;
  string sDpLock, sDpSelectedManager, sSelectedManager, sSystemName;
  bool bLocked, bAddress=pcFGC2_PS_MPC_checkAddresOutput(deviceName);

// 2. Get parameters to call the set button function
  sSystemName = unGenericDpFunctions_getSystemName(deviceName);

  sDpLock = deviceName + ".statusInformation.selectedManager:_lock._original._locked";
  sDpSelectedManager = deviceName + ".statusInformation.selectedManager";
  
  if (dpExists(deviceName))
  {
    iRes = dpGet(sDpLock, bLocked, sDpSelectedManager, sSelectedManager);
  }
  else
  {
    iRes = -1;
  }
  if (iRes < 0) // Errors during dpget
  {
    bLocked = false;
    sSelectedManager = "";
  }
  dsReturnData=makeDynString(bLocked, sSelectedManager, bAddress, "", "", "", "");
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "FGC button FGC2_PS_MPC "+deviceName, "pcFGC2_PS_MPC_UserLoginGetButtonState", deviceName, sType, dsReturnData);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC2_PS_MPC_ButtonDisconnection
/** set the state of the device action button when the device system is disconnected 
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

*/
pcFGC2_PS_MPC_ButtonDisconnection()
{
  setMultiValue("buttonPAL_CHAN_DIVPAM_ID", "enabled", false, 
                  "buttonCONFIG_MODE", "enabled", false, "buttonSelect", "enabled", false);
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "FGC button FGC2_PS_MPC", "pcFGC2_PS_MPC_ButtonDisconnection");
}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC2_PS_MPC_FaceplateStatusRegisterCB
/** faceplate DistributedControl callback of the faceplate status panel
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDp input, the DistributedControl system name DP name
@param bSystemConnected input, the state of the system name
*/
pcFGC2_PS_MPC_FaceplateStatusRegisterCB(string sDp, bool bSystemConnected)
{
  string deviceName, sSystemName;
  int iAction, iRes;
  dyn_string exceptionInfo;
  bool bRemote;
  
  deviceName = unGenericDpFunctions_getDpName($sDpName);
  sSystemName = unGenericDpFunctions_getSystemName(deviceName);
  unDistributedControl_isRemote(bRemote, sSystemName);
  if(bRemote)
    g_bSystemConnected = bSystemConnected;
  else
    g_bSystemConnected = true;

  unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "FGC faceplate FGC2_PS_MPC "+sDp, "pcFGC2_PS_MPC_FaceplateStatusRegisterCB", sDp, deviceName, g_bCallbackConnected, iAction);
  switch (iAction)
    {
    case UN_ACTION_DISCONNECT:
      pcFGC2_PS_MPC_FaceplateStatusDisconnection();
      break;
    case UN_ACTION_DPCONNECT:
      pcFGC2_PS_MPC_FaceplateConnect(deviceName);
      break;
    case UN_ACTION_DPDISCONNECT_DISCONNECT:
      pcFGC2_PS_MPC_FaceplateDisconnect(deviceName);
      pcFGC2_PS_MPC_FaceplateStatusDisconnection();
      break;
    case UN_ACTION_NOTHING:
    default:
      break;
    }
}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC2_PS_MPC_FaceplateConnect
/** dpConnect to the device data
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device DP name
*/
pcFGC2_PS_MPC_FaceplateConnect(string deviceName)
{
  int iRes, iWF;
  dyn_string exceptionInfo;
  string sFrontEnd, sSystFE, sSystemName, sFE, sAppl;
  string sDeviceAlias = unGenericDpFunctions_getAlias(deviceName);

  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "FGC faceplate FGC2_PS_MPC "+deviceName, "pcFGC2_PS_MPC_FaceplateConnect", deviceName, sFrontEnd, g_bUnSystemAlarmPlc);
  g_bUnSystemAlarmPlc = false;
// get the systemName
  sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
  sFrontEnd = unGenericObject_GetPLCNameFromDpName(deviceName);
  g_sFrontEndDp = sFrontEnd;
  
  if(sFrontEnd != ""){
    sSystFE = sSystemName+c_unSystemAlarm_dpPattern+DS_pattern+sFrontEnd;
    sFrontEnd = sSystemName+sFrontEnd;
//DebugN(sSystemName, sFrontEnd);
    if(dpExists(sSystFE)) {
      g_bUnSystemAlarmPlc = true;
    }
  }
    
  if(g_bUnSystemAlarmPlc) {
    pcFGC2_LHC_PC_getConfig(deviceName, sFE, sAppl, g_iBitPosFGC, iWF);
    pcFGC2_LHC_PC_loadTextLabel(PC_STATE_PC_TEXT_LABELDP, g_mSTATE_PC);
    pcFGC2_LHC_PC_loadTextLabel(PC_STATE_OP_TEXT_LABELDP, g_mSTATE_OP);
    g_sBFormat     = dpGetFormat(deviceName + ".ProcessInput.B_MEAS");
    g_sBUnit       = dpGetUnit(deviceName + ".ProcessInput.B_MEAS");
    g_sVFormat     = dpGetFormat(deviceName + ".ProcessInput.V_MEAS");
    g_sVUnit       = dpGetUnit(deviceName + ".ProcessInput.V_MEAS");
    g_sIFormat     = dpGetFormat(deviceName + ".ProcessInput.I_MEAS");
    g_sIUnit       = dpGetUnit(deviceName + ".ProcessInput.I_MEAS");
    
    iRes = dpConnect("pcFGC2_PS_MPC_FaceplateStatusAnimationCB",
                      deviceName + ".ProcessInput.STATE_PC:_online.._invalid",
                       deviceName + ".ProcessInput.STATE_PC",
                       deviceName + ".ProcessInput.STATE_OP",
                       deviceName + ".ProcessInput.ST_FAULTS", 
                       deviceName + ".ProcessInput.ST_WARNINGS", 
                       deviceName + ".ProcessInput.ST_UNLATCHED", 
                       deviceName + ".ProcessInput.B_MEAS",
                       deviceName + ".ProcessInput.V_MEAS",
                       deviceName + ".ProcessInput.B_REF",
                       deviceName + ".ProcessInput.V_REF",
                       deviceName + ".ProcessInput.I_MEAS",
                       deviceName + ".ProcessInput.ST_GTWDEV",
                       sFrontEnd + ".communication.FGC_MASK",
                       sSystFE+".alarm",
                      sSystFE+".enabled"
                      );
                                                        
  }
  else
    pcFGC2_PS_MPC_FaceplateStatusDisconnection();
  g_bCallbackConnected = (iRes >= 0);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC2_PS_MPC_FaceplateDisconnect
/** dpDisconnect to the device data
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device DP name
*/
pcFGC2_PS_MPC_FaceplateDisconnect(string deviceName)
{
  int iRes;
  string sFrontEnd, sSystFE, sSystemName;

  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "FGC faceplate FGC2_PS_MPC "+deviceName, "pcFGC2_PS_MPC_FaceplateDisconnect", deviceName, g_bUnSystemAlarmPlc);
  if(g_bUnSystemAlarmPlc) {
// get the systemName
    sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
    sFrontEnd = unGenericObject_GetPLCNameFromDpName(deviceName);

    sSystFE = sSystemName+c_unSystemAlarm_dpPattern+DS_pattern+sFrontEnd;
    sFrontEnd = sSystemName+sFrontEnd;
//DebugN(sSystemName, sFrontEnd);

    iRes = dpDisconnect("pcFGC2_PS_MPC_FaceplateStatusAnimationCB",
                      deviceName + ".ProcessInput.STATE_PC:_online.._invalid",
                       deviceName + ".ProcessInput.STATE_PC",
                       deviceName + ".ProcessInput.STATE_OP",
                       deviceName + ".ProcessInput.ST_FAULTS", 
                       deviceName + ".ProcessInput.ST_WARNINGS", 
                       deviceName + ".ProcessInput.ST_UNLATCHED", 
                       deviceName + ".ProcessInput.B_MEAS",
                       deviceName + ".ProcessInput.V_MEAS",
                       deviceName + ".ProcessInput.B_REF",
                       deviceName + ".ProcessInput.V_REF",
                       deviceName + ".ProcessInput.I_MEAS",
                       deviceName + ".ProcessInput.ST_GTWDEV",
                       sFrontEnd + ".communication.FGC_MASK",
                       sSystFE+".alarm",
                      sSystFE+".enabled"
                      );
  }
  g_bCallbackConnected = !(iRes >= 0);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC2_PS_MPC_FaceplateStatusAnimationCB
/** callback function on the device data
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDp1 input, the DPE1 
@param blobData input, device data
@param sDpInvalid input, the invalid DPE 
@param bInvalid input, data validity
@param sDpSTATE_PC input, the STATE_PC DPE 
@param iSTATE_PC input, the STATE_PC value
@param sDpSTATE_OP input, the STATE_OP DPE 
@param iSTATE_OP input, the STATE_OP value
@param sDpST_FAULTS input, the ST_FAULTS DPE 
@param iST_FAULTS input, the ST_FAULTS value
@param sDpST_WARNINGS input, the ST_WARNINGS DPE 
@param iST_WARNINGS input, the ST_WARNINGS value
@param sDpST_UNLATCHED input, the ST_UNLATCHED DPE 
@param iST_UNLATCHED input, the ST_UNLATCHED value
@param sDpB_MEAS input, the B_MEAS DPE 
@param fB_MEAS input, the B_MEAS value
@param sDpV_MEAS input, the V_MEAS DPE 
@param fV_MEAS input, the V_MEAS value
@param sDpB_REF input, the B_REF DPE 
@param fB_REF input, the B_REF value
@param sDpV_REF input, the V_REF DPE 
@param fV_REF input, the V_REF value
@param sDpI_MEAS input, the I_MEAS DPE 
@param fI_MEAS input, the I_MEAS value
@param sDpDevice input, the ST_GTWDEV DPE 
@param bStDevice input, the ST_GTWDEV value
@param sDpFGC_MASK input, the GTW FGC_MASK DPE 
@param b32FGC_MASK input, the GTW FGC_MASK value
@param sDpFESystemIntegrityAlarmValue input, front-end device system integrity alarm DPE 
@param iFESystemIntegrityAlarmValue input, front-end device system integrity alarm value
@param sDpFESystemIntegrityAlarmEnabled input, front-end device system integrity enable DPE
@param bFESystemIntegrityAlarmEnabled input, front-end device system integrity enable value
*/
pcFGC2_PS_MPC_FaceplateStatusAnimationCB(string sDpInvalid, bool bInvalid,
                    string sDp_STATE_PC, int iSTATE_PC,  
                    string sDp_STATE_OP, int iSTATE_OP,
                    string sDp_ST_FAULTS, int iST_FAULTS,
                    string sDp_ST_WARNINGS, int iST_WARNINGS,
                    string sDp_ST_UNLATCHED, int iST_UNLATCHED,
                    string sDp_B_MEAS, float fB_MEAS,
                    string sDp_V_MEAS, float fV_MEAS,
                    string sDp_B_REF, float fB_REF,
                    string sDp_V_REF, float fV_REF,
                    string sDp_I_MEAS, float fI_MEAS,
                    string sDpDevice, bool bStDevice,
                    string sDpFGC_MASK, bit32 b32FGC_MASK,
                    string sDpFESystemIntegrityAlarmValue, int iFESystemIntegrityAlarmValue, 
                    string sDpFESystemIntegrityAlarmEnabled, bool bFESystemIntegrityAlarmEnabled)
{
  int iAlarm=c_unSystemIntegrity_no_alarm_value;
  string sSTATE_PCLetter, sSTATE_OPLetter, sSTATE_PCColor, sSTATE_OPColor, sOkColor, sBadColor;
  bool bInvalidData;
  
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "FGC faceplate FGC_PS_MPC "+sDpInvalid, "pcFGC2_PS_MPC_FaceplateStatusAnimationCB", sDpInvalid, bInvalid, g_bSystemConnected, 
                               iFESystemIntegrityAlarmValue, bFESystemIntegrityAlarmEnabled);
  
  // if Front-end problem or ST_FIP problems
  if(!getBit(b32FGC_MASK, g_iBitPosFGC))
  {
    // set state OP to unknown and state PC to OFFLINE
    iAlarm = c_unSystemIntegrity_alarm_value_level1;
    iSTATE_PC = PC_TEXT_LABEL_FGC_OFFLINE;
    iSTATE_OP = PC_TEXT_LABEL_FGC_OFFLINE;
    bInvalidData = TRUE;
  }
  else if(!bStDevice || (iFESystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value))
  {
    iAlarm = c_unSystemIntegrity_alarm_value_level1;
    bInvalidData = TRUE;
  }
    
  bInvalidData = bInvalidData | bInvalid;
    
  if (bInvalidData)
  {
    sOkColor = "unDataNotValid";
    sBadColor = "unDataNotValid";    
  }
  else
  {
    sOkColor = "unFaceplate_OperationModeActive";
    sBadColor = "unFaceplate_OperationModeActive";
  }
      
  if(g_bSystemConnected)
    pcFGC2_LHC_PC_FaceplateStatus_Header(g_sFrontEndDp, iAlarm, bFESystemIntegrityAlarmEnabled, bInvalidData);
    
  // get STATE_PC, STATE_OP
  if(mappingHasKey(g_mSTATE_PC, iSTATE_PC))
  {
    sSTATE_PCLetter = g_mSTATE_PC[iSTATE_PC][PC_TEXT_LABEL_TEXT];
    if(bInvalidData)
      sSTATE_PCColor = "unDataNotValid";
    else
      sSTATE_PCColor = g_mSTATE_PC[iSTATE_PC][PC_TEXT_LABEL_COLOR];
  }
  else
  {
    sSTATE_PCLetter = PC_FGC_UNKNOWN_VALUE;
    sSTATE_PCColor = "unDataNotUpToDate";
  }
      
  if(mappingHasKey(g_mSTATE_OP, iSTATE_OP))
  {
    sSTATE_OPLetter = g_mSTATE_OP[iSTATE_OP][PC_TEXT_LABEL_TEXT];
    if(bInvalidData)
      sSTATE_OPColor = "unDataNotValid";
    else
      sSTATE_OPColor = g_mSTATE_OP[iSTATE_OP][PC_TEXT_LABEL_COLOR];
  }
  else
  {
    sSTATE_OPLetter = PC_FGC_UNKNOWN_VALUE;
    sSTATE_OPColor = "unDataNotUpToDate";
  }


  if(g_bSystemConnected)
    setMultiValue(  "STATE_PC", "text", sSTATE_PCLetter, 
            "STATE_PC", "foreCol", sSTATE_PCColor,
            "STATE_PC", "backCol", "unSynopticBackground",
            "STATE_OP", "text", sSTATE_OPLetter, 
            "STATE_OP", "foreCol", sSTATE_OPColor,
            "STATE_OP", "backCol", "unSynopticBackground");
    
  if(g_bSystemConnected)
    unGenericObject_DisplayValue(g_sBFormat, g_sBUnit, fB_MEAS, "B_MEAS", "unDisplayValue_Status", bInvalidData);
  if(g_bSystemConnected)
    unGenericObject_DisplayValue(g_sVFormat, g_sVUnit, fV_MEAS, "V_MEAS", "unDisplayValue_Status", bInvalidData);
  if(g_bSystemConnected)
    unGenericObject_DisplayValue(g_sBFormat, g_sBUnit, fB_REF, "B_REF", "unDisplayValue_Status", bInvalidData);
  if(g_bSystemConnected)
    unGenericObject_DisplayValue(g_sVFormat, g_sVUnit, fV_REF, "V_REF", "unDisplayValue_Status", bInvalidData);
  if(g_bSystemConnected)
    unGenericObject_DisplayValue(g_sIFormat, g_sIUnit, fI_MEAS, "I_MEAS", "unDisplayValue_Status", bInvalidData);
    
  //ST_FAULTS
  if(g_bSystemConnected)
    unGenericObject_ColoredSquareAnimate("ST_FAULTS_FB_LOOP", pcFGC2_LHC_PC_FaceplateStatus_getBitState("FGC2_PS_MPC_ST_FAULTS", iST_FAULTS, "FB_LOOP"), true, sOkColor, sBadColor, bInvalidData);
  if(g_bSystemConnected)
    unGenericObject_ColoredSquareAnimate("ST_FAULTS_FGC_HW", pcFGC2_LHC_PC_FaceplateStatus_getBitState("FGC2_PS_MPC_ST_FAULTS", iST_FAULTS, "FGC_HW"), true, sOkColor, sBadColor, bInvalidData);
  if(g_bSystemConnected)
    unGenericObject_ColoredSquareAnimate("ST_FAULTS_FGC_STATE", pcFGC2_LHC_PC_FaceplateStatus_getBitState("FGC2_PS_MPC_ST_FAULTS", iST_FAULTS, "FGC_STATE"), true, sOkColor, sBadColor, bInvalidData);
  if(g_bSystemConnected)
    unGenericObject_ColoredSquareAnimate("ST_FAULTS_I_MEAS", pcFGC2_LHC_PC_FaceplateStatus_getBitState("FGC2_PS_MPC_ST_FAULTS", iST_FAULTS, "I_MEAS"), true, sOkColor, sBadColor, bInvalidData);
  if(g_bSystemConnected)
    unGenericObject_ColoredSquareAnimate("ST_FAULTS_LIMITS", pcFGC2_LHC_PC_FaceplateStatus_getBitState("FGC2_PS_MPC_ST_FAULTS", iST_FAULTS, "LIMITS"), true, sOkColor, sBadColor, bInvalidData);
  if(g_bSystemConnected)
    unGenericObject_ColoredSquareAnimate("ST_FAULTS_MPC_LINK", pcFGC2_LHC_PC_FaceplateStatus_getBitState("FGC2_PS_MPC_ST_FAULTS", iST_FAULTS, "MPC_LINK"), true, sOkColor, sBadColor, bInvalidData);
  if(g_bSystemConnected)
    unGenericObject_ColoredSquareAnimate("ST_FAULTS_VS_EXTINTLOCK", pcFGC2_LHC_PC_FaceplateStatus_getBitState("FGC2_PS_MPC_ST_FAULTS", iST_FAULTS, "VS_EXTINTLOCK"), true, sOkColor, sBadColor, bInvalidData);
  if(g_bSystemConnected)
    unGenericObject_ColoredSquareAnimate("ST_FAULTS_VS_FAULT", pcFGC2_LHC_PC_FaceplateStatus_getBitState("FGC2_PS_MPC_ST_FAULTS", iST_FAULTS, "VS_FAULT"), true, sOkColor, sBadColor, bInvalidData);
  if(g_bSystemConnected)
    unGenericObject_ColoredSquareAnimate("ST_FAULTS_VS_RUN_TO", pcFGC2_LHC_PC_FaceplateStatus_getBitState("FGC2_PS_MPC_ST_FAULTS", iST_FAULTS, "VS_RUN_TO"), true, sOkColor, sBadColor, bInvalidData);
  if(g_bSystemConnected)
    unGenericObject_ColoredSquareAnimate("ST_FAULTS_VS_STATE", pcFGC2_LHC_PC_FaceplateStatus_getBitState("FGC2_PS_MPC_ST_FAULTS", iST_FAULTS, "VS_STATE"), true, sOkColor, sBadColor, bInvalidData);
    
  //ST_WARNINGS
  if(g_bSystemConnected)
    unGenericObject_ColoredSquareAnimate("ST_WARNINGS_B_MEAS", pcFGC2_LHC_PC_FaceplateStatus_getBitState("FGC2_PS_MPC_ST_WARNINGS", iST_WARNINGS, "B_MEAS"), true, sOkColor, sBadColor, bInvalidData);
  if(g_bSystemConnected)
    unGenericObject_ColoredSquareAnimate("ST_WARNINGS_CONFIG", pcFGC2_LHC_PC_FaceplateStatus_getBitState("FGC2_PS_MPC_ST_WARNINGS", iST_WARNINGS, "CONFIG"), true, sOkColor, sBadColor, bInvalidData);
  if(g_bSystemConnected)
    unGenericObject_ColoredSquareAnimate("ST_WARNINGS_CYC_START", pcFGC2_LHC_PC_FaceplateStatus_getBitState("FGC2_PS_MPC_ST_WARNINGS", iST_WARNINGS, "CYC_START"), true, sOkColor, sBadColor, bInvalidData);
  if(g_bSystemConnected)
    unGenericObject_ColoredSquareAnimate("ST_WARNINGS_DIDT_MISMATCH", pcFGC2_LHC_PC_FaceplateStatus_getBitState("FGC2_PS_MPC_ST_WARNINGS", iST_WARNINGS, "DIDT_MISMATCH"), true, sOkColor, sBadColor, bInvalidData);
  if(g_bSystemConnected)
    unGenericObject_ColoredSquareAnimate("ST_WARNINGS_FB_LOOP", pcFGC2_LHC_PC_FaceplateStatus_getBitState("FGC2_PS_MPC_ST_WARNINGS", iST_WARNINGS, "FB_LOOP"), true, sOkColor, sBadColor, bInvalidData);
  if(g_bSystemConnected)
    unGenericObject_ColoredSquareAnimate("ST_WARNINGS_FGC_HW", pcFGC2_LHC_PC_FaceplateStatus_getBitState("FGC2_PS_MPC_ST_WARNINGS", iST_WARNINGS, "FGC_HW"), true, sOkColor, sBadColor, bInvalidData);
  if(g_bSystemConnected)
    unGenericObject_ColoredSquareAnimate("ST_WARNINGS_FGC_PSU", pcFGC2_LHC_PC_FaceplateStatus_getBitState("FGC2_PS_MPC_ST_WARNINGS", iST_WARNINGS, "FGC_PSU"), true, sOkColor, sBadColor, bInvalidData);
  if(g_bSystemConnected)
    unGenericObject_ColoredSquareAnimate("ST_WARNINGS_I_MEAS", pcFGC2_LHC_PC_FaceplateStatus_getBitState("FGC2_PS_MPC_ST_WARNINGS", iST_WARNINGS, "I_MEAS"), true, sOkColor, sBadColor, bInvalidData);
  if(g_bSystemConnected)
    unGenericObject_ColoredSquareAnimate("ST_WARNINGS_I_RMS_LIM", pcFGC2_LHC_PC_FaceplateStatus_getBitState("FGC2_PS_MPC_ST_WARNINGS", iST_WARNINGS, "I_RMS_LIM"), true, sOkColor, sBadColor, bInvalidData);
  if(g_bSystemConnected)
    unGenericObject_ColoredSquareAnimate("ST_WARNINGS_MPC_LINK", pcFGC2_LHC_PC_FaceplateStatus_getBitState("FGC2_PS_MPC_ST_WARNINGS", iST_WARNINGS, "MPC_LINK"), true, sOkColor, sBadColor, bInvalidData);
  if(g_bSystemConnected)
    unGenericObject_ColoredSquareAnimate("ST_WARNINGS_PPM_CYC_LEN", pcFGC2_LHC_PC_FaceplateStatus_getBitState("FGC2_PS_MPC_ST_WARNINGS", iST_WARNINGS, "PPM_CYC_LEN"), true, sOkColor, sBadColor, bInvalidData);
  if(g_bSystemConnected)
    unGenericObject_ColoredSquareAnimate("ST_WARNINGS_PPM_CYC_USER", pcFGC2_LHC_PC_FaceplateStatus_getBitState("FGC2_PS_MPC_ST_WARNINGS", iST_WARNINGS, "PPM_CYC_USER"), true, sOkColor, sBadColor, bInvalidData);
  if(g_bSystemConnected)
    unGenericObject_ColoredSquareAnimate("ST_WARNINGS_REF_RATE_LIM", pcFGC2_LHC_PC_FaceplateStatus_getBitState("FGC2_PS_MPC_ST_WARNINGS", iST_WARNINGS, "REF_RATE_LIM"), true, sOkColor, sBadColor, bInvalidData);
  if(g_bSystemConnected)
    unGenericObject_ColoredSquareAnimate("ST_WARNINGS_TEMPERATURE", pcFGC2_LHC_PC_FaceplateStatus_getBitState("FGC2_PS_MPC_ST_WARNINGS", iST_WARNINGS, "TEMPERATURE"), true, sOkColor, sBadColor, bInvalidData);
  if(g_bSystemConnected)
    unGenericObject_ColoredSquareAnimate("ST_WARNINGS_VS_GAIN", pcFGC2_LHC_PC_FaceplateStatus_getBitState("FGC2_PS_MPC_ST_WARNINGS", iST_WARNINGS, "VS_GAIN"), true, sOkColor, sBadColor, bInvalidData);
  if(g_bSystemConnected)
    unGenericObject_ColoredSquareAnimate("ST_WARNINGS_VS_SIM", pcFGC2_LHC_PC_FaceplateStatus_getBitState("FGC2_PS_MPC_ST_WARNINGS", iST_WARNINGS, "VS_SIM"), true, sOkColor, sBadColor, bInvalidData);
    
  //ST_UNLATCHED
  if(g_bSystemConnected)
    unGenericObject_ColoredSquareAnimate("ST_UNLATCHED_ABORT_EVENT", pcFGC2_LHC_PC_FaceplateStatus_getBitState("FGC2_PS_MPC_ST_UNLATCHED", iST_UNLATCHED, "ABORT_EVENT"), true, sOkColor, sBadColor, bInvalidData);
  if(g_bSystemConnected)
    unGenericObject_ColoredSquareAnimate("ST_UNLATCHED_I_MEAS_DIFF", pcFGC2_LHC_PC_FaceplateStatus_getBitState("FGC2_PS_MPC_ST_UNLATCHED", iST_UNLATCHED, "I_MEAS_DIFF"), true, sOkColor, sBadColor, bInvalidData);
  if(g_bSystemConnected)
    unGenericObject_ColoredSquareAnimate("ST_UNLATCHED_LOW_CURRENT", pcFGC2_LHC_PC_FaceplateStatus_getBitState("FGC2_PS_MPC_ST_UNLATCHED", iST_UNLATCHED, "LOW_CURRENT"), true, sOkColor, sBadColor, bInvalidData);
  if(g_bSystemConnected)
    unGenericObject_ColoredSquareAnimate("ST_UNLATCHED_NOMINAL_LOAD", pcFGC2_LHC_PC_FaceplateStatus_getBitState("FGC2_PS_MPC_ST_UNLATCHED", iST_UNLATCHED, "NOMINAL_LOAD"), true, sOkColor, sBadColor, bInvalidData);
  if(g_bSystemConnected)
    unGenericObject_ColoredSquareAnimate("ST_UNLATCHED_PC_PERMIT", pcFGC2_LHC_PC_FaceplateStatus_getBitState("FGC2_PS_MPC_ST_UNLATCHED", iST_UNLATCHED, "PC_PERMIT"), true, sOkColor, sBadColor, bInvalidData);
  if(g_bSystemConnected)
    unGenericObject_ColoredSquareAnimate("ST_UNLATCHED_POST_MORTEM", pcFGC2_LHC_PC_FaceplateStatus_getBitState("FGC2_PS_MPC_ST_UNLATCHED", iST_UNLATCHED, "POST_MORTEM"), true, sOkColor, sBadColor, bInvalidData);
  if(g_bSystemConnected)
    unGenericObject_ColoredSquareAnimate("ST_UNLATCHED_PPM_LOG_READY", pcFGC2_LHC_PC_FaceplateStatus_getBitState("FGC2_PS_MPC_ST_UNLATCHED", iST_UNLATCHED, "PPM_LOG_READY"), true, sOkColor, sBadColor, bInvalidData);
  if(g_bSystemConnected)
    unGenericObject_ColoredSquareAnimate("ST_UNLATCHED_START_EVENT", pcFGC2_LHC_PC_FaceplateStatus_getBitState("FGC2_PS_MPC_ST_UNLATCHED", iST_UNLATCHED, "START_EVENT"), true, sOkColor, sBadColor, bInvalidData);
  if(g_bSystemConnected)
    unGenericObject_ColoredSquareAnimate("ST_UNLATCHED_SYNC_PLEASE", pcFGC2_LHC_PC_FaceplateStatus_getBitState("FGC2_PS_MPC_ST_UNLATCHED", iST_UNLATCHED, "SYNC_PLEASE"), true, sOkColor, sBadColor, bInvalidData);
  if(g_bSystemConnected)
    unGenericObject_ColoredSquareAnimate("ST_UNLATCHED_VS_POWER_ON", pcFGC2_LHC_PC_FaceplateStatus_getBitState("FGC2_PS_MPC_ST_UNLATCHED", iST_UNLATCHED, "VS_POWER_ON"), true, sOkColor, sBadColor, bInvalidData);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC2_PS_MPC_FaceplateStatusDisconnection
/** set the faceplate when the device system is disconnected 
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

*/
pcFGC2_PS_MPC_FaceplateStatusDisconnection()
{
// 1. animate color boxes

  unGenericObject_ColoredSquareDisconnect(makeDynString("ST_FAULTS_FB_LOOP", "ST_FAULTS_FGC_HW", 
                                                         "ST_FAULTS_FGC_STATE", "ST_FAULTS_I_MEAS",
                                                         "ST_FAULTS_LIMITS", "ST_FAULTS_MPC_LINK",
                                                         "ST_FAULTS_VS_EXTINTLOCK", "ST_FAULTS_VS_FAULT",
                                                         "ST_FAULTS_VS_RUN_TO", "ST_FAULTS_VS_STATE",
                                                         "ST_WARNINGS_B_MEAS", "ST_WARNINGS_CONFIG",
                                                         "ST_WARNINGS_CYC_START", "ST_WARNINGS_DIDT_MISMATCH",
                                                         "ST_WARNINGS_FB_LOOP", "ST_WARNINGS_FGC_HW",
                                                         "ST_WARNINGS_FGC_PSU", "ST_WARNINGS_I_MEAS", 
                                                         "ST_WARNINGS_I_RMS_LIM", "ST_WARNINGS_MPC_LINK", 
                                                         "ST_WARNINGS_PPM_CYC_LEN", "ST_WARNINGS_PPM_CYC_USER", 
                                                         "ST_WARNINGS_REF_RATE_LIM", "ST_WARNINGS_TEMPERATURE", 
                                                         "ST_WARNINGS_VS_GAIN", "ST_WARNINGS_VS_SIM",
                                                         "ST_UNLATCHED_ABORT_EVENT", "ST_UNLATCHED_I_MEAS_DIFF", 
                                                         "ST_UNLATCHED_LOW_CURRENT", "ST_UNLATCHED_NOMINAL_LOAD", 
                                                         "ST_UNLATCHED_PC_PERMIT", "ST_UNLATCHED_POST_MORTEM", 
                                                         "ST_UNLATCHED_PPM_LOG_READY", "ST_UNLATCHED_START_EVENT",
                                                         "ST_UNLATCHED_SYNC_PLEASE", "ST_UNLATCHED_VS_POWER_ON"));
               
  // 2. animate Current and Voltage
  unGenericObject_DisplayValueDisconnect(makeDynString("I_MEAS", "B_MEAS", "V_MEAS", "B_REF", "V_REF"));
  
  setMultiValue("STATE_PC", "color", "unDataNoAccess",
                "STATE_OP", "color", "unDataNoAccess");
                
// 2. animate PLC Connection
  pcFGC2_LHC_PC_FaceplateStatus_HeaderDisconnect();
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "FGC faceplate FGC_PS_MPC", "pcFGC2_PS_MPC_FaceplateStatusDisconnection");
}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC2_PS_MPC_OpenSetPanel
/** Open a panel
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sFileName input, the panel file name to open 
@param sPanelName input, the panel name
@param sDeviceDpName input, the device DP Name 
*/
pcFGC2_PS_MPC_OpenSetPanel(string sFileName, string sPanelName, string sDeviceDpName)
{
  int x, y;
  
  getCursorPosition(x, y);
  unGraphicalFrame_ChildPanelOn(sFileName, sPanelName, 
                                     makeDynString("$sDeviceDpName:"+sDeviceDpName), x, y);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC2_PS_MPC_SetFGCValue
/** Set the FGC2_PS_MPC value
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device DP Name
@param sType input, the type of request
@param sDpe input, the device DPE Name 
@param aValue input, the device DPE value 
@param exInfo output, exception are returned here
*/
pcFGC2_PS_MPC_SetFGCValue(string deviceName, string sType, string sDpe, anytype aValue, dyn_string &exInfo)
{
  string alias = unGenericDpFunctions_getAlias(deviceName);	
  dyn_string exceptionInfoTemp;
  
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "pops set "+deviceName, "popsCVTDO_setValue", deviceName);

  unSendMessage_toAllUsers(alias + " --> "+ sType + " requested", exInfo);

  pcFGC2_LHC_PC_setFGCValue(deviceName + sDpe, aValue, exInfo);
  
  if(dynlen(exInfo) > 0)	// error Set
  {
    unSendMessage_toAllUsers(alias + " --> "+ sType + " requested FAILED",exceptionInfoTemp);
    if(dynlen(exceptionInfoTemp) > 0)
    {
      fwExceptionHandling_display(exceptionInfoTemp);
    }
  }
  else
  {
    unSendMessage_toAllUsers(alias + " --> "+ sType + " requested OK",exceptionInfoTemp);
    if(dynlen(exceptionInfoTemp) > 0)
    {
      fwExceptionHandling_display(exceptionInfoTemp);
    }
  }
}

//---------------------------------------------------------------------------------------------------------------------------------------

//@}

  



