/**@name LIBRARY: unSystemIntegrity.ctl

@author: Herve Milcent (AB-CO)

Creation Date: 12/07/2002

Modification History: 
  22/02/2012: Herve
  - IS-719  unSystemIntegrity  Manager manager from UI (only): being able to kill a manager even during the init phase  
 
  05/09/2011: Herve
  - IS-598: system name and component version added in the MessageText log and in the diagnostic 
  - IS-612  Installation  postinstall script: remove or stop managers not requested if no in the console
   
  - IS-555: system integrity configuration panel: alow remote access, restart unicos_scripts
  
  - IS-373: split the unCore libs, files, etc, used in unSystemIntegrity
  
  11/05/2011: Herve
  -IS-529: Import: delete devices and front-end, remove the device and any associated DP from all the mail/SMS category and application
  
  25/11/2010: Herve
  - IS-462: remove the systemAlarm DPE from the mail/sms setting when the systemAlarm is deleted
  
  16/08/2010: Herve
    - IS-380: 
    modified function: unSystemIntergity_manageManagerperHostname, bug fix: 
      - when the started manager has no argument, the search exclude it
      - restart, reset start counter, seconds to kill are set by default instead of being read and restored in case of set manual and set always 
    add two new constants for DEBUG option and extended diagnostic.
    
  26/01/2010: Herve
    - add new function: _unSystemIntegrity_createSystemAlarmBasic
    create systemAlarm with more than 1 alarm level.
    
  23/09/2009: Herve
    - add constant for the unSystemIntegrity_computeStatistics
    - new function:
    _unSystemIntegrity_createScriptScheduler: creates a scheduler to start a script
    _unSystemIntegrity_startScheduler: start the scheduler
    
  20/04/2009: Herve
    - _unSystemIntegrity_createScheduler: add sSystemName as extra parameter of the function in order to
    create the scheduler on the remote system
    
	01/07/2008: Frederic
		- functions: unSystemIntergity_manageManagerperHostname() and unSystemIntegrity_getState()
		the problem with archive -num 10 (!= -num 1) fixed

	10/12/2007: Herve
		- new function: unSystemIntegrity_getState
		
	13/07/2007: Frederic
		- add unSystemIntergity_getManagerStateperHostname and unSystemIntergity_manageManagerperHostname

	02/03/2005: Herve
		- unSystemIntergity_manageManager: add del manager, set manager to manual mode
		
	26/02/2005: Herve
		- in unSystemIntergity_manageManager, kill and stop the manager only if it is in running or blocked mode.
		
	22/02/2005: Herve
		- add functions unSystemIntergity_getManagerState unSystemIntergity_manageManager
		
	15/02/2005: Herve
		- add a new function: unSystemIntegrity_createFrontEndSystemAlarm: that creates a _UnSystemAlarm with negative behaviour Ok>=10, error <10
		- modification of the signature of _unSystemIntegrity_createSystemAlarm to allow the choice of the alarm logic
		
	02/07/2004: Herve
		version 3.0
		move c_unSystemIntegrity_defaultDPECheckDelay to unSystemIntegrity_DPE.ctl

	16/04/2004: Herve
		move functions from the unSystemIntegrity script to the lib.
		move PLC function to systemIntegrity_unicos.ctl

	16/03/2004: Fredric
		add the function: _unSystemIntegrity_createScheduler
	
	26/01/2004: Herve
		add constant for check dpe value

	15/09/2003: Herve
		add the functions unSystemIntegrity_sendCmdToPLC and unSystemIntegrity_synchronizedTimeDateOfPlc

	07/10/2003: Herve
		move systemIntegrity from unicosObjects to unCore

version 1.0

External Functions: 
	
Internal Functions: 
	. _unSystemIntegrity_modifyApplicationAlertList: Add/remove a dp in the list of dp alert of the application DP
	. _unSystemIntegrity_createSystemAlarm: create the dp and set the alert hdl
	
Purpose: 
	Utility library for the System Integrity component.
	 	
Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. global variable: the following variables are global to the script
	. data point type needed: _UnSystemAlarm, _UnSystemIntegrity, _UnPlc
	. data point: 
	. PVSS version: 3.0 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/

//--------------------------------------------------------------------------------------------------------------------------------
// constant declaration
//------------------
// SYSTEM INTEGRITY
//------------------
const string c_unSystemAlarm_dpPattern = "_unSystemAlarm_";
const string c_unSystemAlarm_dpType = "_UnSystemAlarm";
const string UN_SYSTEMINTEGRITY_EXTENSION = "_systemIntegrityInfo";
const string UN_SYSTEMINTEGRITY_DPTYPE = "_UnSystemIntegrity";

const int UN_SYSTEMINTEGRITY_ADD = 1;
const int UN_SYSTEMINTEGRITY_DELETE = 2;
const int UN_SYSTEMINTEGRITY_ENABLE = 3;
const int UN_SYSTEMINTEGRITY_DISABLE = 4;
const int UN_SYSTEMINTEGRITY_DIAGNOSTIC = 5;
const int UN_SYSTEMINTEGRITY_EXTENDED_DIAGNOSTIC = 10;
const int UN_SYSTEMINTEGRITY_DEBUG = 11;
const int UN_SYSTEMINTEGRITY_VERSION = 12;

const int c_unSystemIntegrity_alarm_value_level1 = 10;
const int c_unSystemIntegrity_no_alarm_value = 0;
const string UN_SYSTEMINTEGRITY_applicationDPName = "_unApplication.applicationName";
const string UN_SYSTEMINTEGRITY_OK = "Ok";
const string UN_SYSTEMINTEGRITY_BAD = "Bad";

const int UN_SYSTEMINTEGRITY_MAX_FUNCTION = 3;
const int UN_SYSTEMINTEGRITY_FUNCTION_INITIALIZE = 1;
const int UN_SYSTEMINTEGRITY_FUNCTION_HANDLE_COMMAND = 2;
const int UN_SYSTEMINTEGRITY_FUNCTION_HANDLE_DATA = 3;

const string c_systemIntegrity_noFwAccessControl_userAdmin = "root";

const string UN_ARCHIVE = "_ArchivDisk.FreeKB";
const string UN_MEMORY = "_MemoryCheck.FreeKB";

const string SYSTEMINTEGRITY_STOP_COMMAND = "STOP";
const string SYSTEMINTEGRITY_PMON_STOP_COMMAND = "SINGLE_MGR:STOP ";
const string SYSTEMINTEGRITY_KILL_COMMAND = "KILL";
const string SYSTEMINTEGRITY_PMON_KILL_COMMAND = "SINGLE_MGR:KILL ";
const string SYSTEMINTEGRITY_START_COMMAND = "START";
const string SYSTEMINTEGRITY_PMON_START_COMMAND = "SINGLE_MGR:START ";
const string SYSTEMINTEGRITY_PMON_QUERY_COMMAND = "MGRLIST:STATI";
const string SYSTEMINTEGRITY_DEL_COMMAND = "DEL";
const string SYSTEMINTEGRITY_PMON_DEL_COMMAND = "SINGLE_MGR:DEL ";
const string SYSTEMINTEGRITY_SETMANUAL_COMMAND = "SETMANUAL";
const string SYSTEMINTEGRITY_SETALWAYS_COMMAND = "SETALWAYS";
const string SYSTEMINTEGRITY_PMON_SETMANUAL_COMMAND = "SINGLE_MGR:PROP_PUT ";
const int SYSTEMINTEGRITY_PMON_UNKNOWN_VALUE = -1;
const int SYSTEMINTEGRITY_MANAGER_STOPPED = 0;
const int SYSTEMINTEGRITY_MANAGER_INIT = 1;
const int SYSTEMINTEGRITY_MANAGER_RUNNING = 2;
const int SYSTEMINTEGRITY_MANAGER_BLOCKED = 3;
const string SYSTEMINTEGRITY_MANAGER_MANUAL = "manual";
const string SYSTEMINTEGRITY_MANAGER_ALWAYS = "always";

const string USERNAME="";
const string PASSWORD="";

const string SYSTEMINTEGRITY_COMPUTESTATISTICS = "unSystemIntegrity_computeStatistics";

//@{
// unSystemIntegrity_createSystemAlarm
/**
Purpose:
This creates the dp and sets the alert hdl.

	@param sDp: string, input, data point name,
	@param sType: string, input, type of System Alarm 
	@param description: string, input, description of the dp to create
	@param exceptionInfo: dyn_string, output, exception are returned here

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/
unSystemIntegrity_createSystemAlarm(string sDp, string sType, string description, dyn_string &exceptionInfo)
{
	string alertText, dp;
	bool bCreate = false;
	int res;
	string arName;
	int arNumber;
	
// remove the system name
	res = strpos(sDp, ":");
	if(res >= 0)
		dp = substr(sDp, res+1, strlen(sDp));
	else
		dp = sDp;
		
	_unSystemIntegrity_createSystemAlarm(c_unSystemAlarm_dpPattern+sType+dp,  
												description, true,
												exceptionInfo);
}

// unSystemIntegrity_createFrontEndSystemAlarm
/**
Purpose:
This creates the dp and sets the alert hdl.

	@param sDp: string, input, data point name,
	@param sType: string, input, type of System Alarm 
	@param description: string, input, description of the dp to create
	@param bOkRange: bool, input, true=alarm if value >10, false=alarm if value <=10
	@param exceptionInfo: dyn_string, output, exception are returned here

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/
unSystemIntegrity_createFrontEndSystemAlarm(string sDp, string sType, string description, bool bOkRange, dyn_string &exceptionInfo)
{
	string alertText, dp;
	bool bCreate = false;
	int res;
	string arName;
	int arNumber;
	
// remove the system name
	res = strpos(sDp, ":");
	if(res >= 0)
		dp = substr(sDp, res+1, strlen(sDp));
	else
		dp = sDp;
		
	_unSystemIntegrity_createSystemAlarm(c_unSystemAlarm_dpPattern+sType+dp, 
												description, bOkRange, 
												exceptionInfo);
}

// _unSystemIntegrity_modifyApplicationAlertList
/**
Purpose:
Add/remove a dp in the list of dp alert of the application DP.

	@param sDp: string, input, string to find in the list
	@param bAdd: bool, input, true = add in the list of alert, false = remove from the list of alert
	@param exceptionInfo: dyn_string, output, exception are returned here

Usage: Internal

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/
_unSystemIntegrity_modifyApplicationAlertList(string sDp, bool bAdd, dyn_string &exceptionInfo)
{
	dyn_string alertText, alertPanelParameter, dp_list;
	string alertPanel, sHelpPanel, dp;
	int pos;
  dyn_string dsTemp;
  int i, len;
  bool bSet;
	
	dp=dpSubStr(sDp, DPSUB_SYS_DP);
  if(dpExists(sDp))
  {
    	if(dpExists(UN_SYSTEMINTEGRITY_applicationDPName)) {
    		dpGet(UN_SYSTEMINTEGRITY_applicationDPName+":_alert_hdl.._text0", alertText[1], 
    						UN_SYSTEMINTEGRITY_applicationDPName+":_alert_hdl.._text1", alertText[2],
    						UN_SYSTEMINTEGRITY_applicationDPName+":_alert_hdl.._panel",alertPanel,
    						UN_SYSTEMINTEGRITY_applicationDPName+":_alert_hdl.._panel_param",alertPanelParameter,
    						UN_SYSTEMINTEGRITY_applicationDPName+":_alert_hdl.._help",sHelpPanel,
    						UN_SYSTEMINTEGRITY_applicationDPName+":_alert_hdl.._dp_list", dp_list);
    	//DebugN(pos, dp);
    		if(bAdd) {
    		  pos = dynContains(dp_list, dp+".alarm");
    			if(pos <= 0){
    				dynAppend(dp_list, dp+".alarm");
    				fwAlertConfig_modifySummary(UN_SYSTEMINTEGRITY_applicationDPName, alertText, dp_list, 
    													alertPanel, alertPanelParameter, sHelpPanel, exceptionInfo);
          unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "unSystemIntegrity Application", "_unSystemIntegrity_modifyApplicationAlertList adding "+dp);
    			}
    		}
    		else {
          bSet = false;
          dsTemp = dynPatternMatch("*"+dp+"*", dp_list);
          len = dynlen(dsTemp);
          for(i=1;i<=len;i++)
          {
            pos = dynContains(dp_list, dsTemp[i]);
            if(pos > 0) {
    				    dynRemove(dp_list, pos);
                bSet = true;
            }
          }
          if(bSet) {
      				fwAlertConfig_modifySummary(UN_SYSTEMINTEGRITY_applicationDPName, alertText, dp_list, 
      													alertPanel, alertPanelParameter, sHelpPanel, exceptionInfo);
          unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "unSystemIntegrity Application", "_unSystemIntegrity_modifyApplicationAlertList removing all instance of "+dp, dp_list);
          }
    		}
    	}
    if(!bAdd)
      unProcessAlarm_removeDpFromList(dp, exceptionInfo);    
  }
  else
    fwException_raise(exceptionInfo,"ERROR","DP: "+sDp+" does not exists","");
}

// _unSystemIntegrity_createSystemAlarm
/**
Purpose:
This creates the dp and sets the alert hdl.

	@param sDp: string, input, data point name
	@param description: string, input, description of the dp
	@param bOkRange: bool, input, true=alarm if value >10, false=alarm if value <=10
	@param exceptionInfo: dyn_string, output, exception are returned here

Usage: Internal

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/
_unSystemIntegrity_createSystemAlarm(string sDp, string description, bool bOkRange, dyn_string &exceptionInfo)
{
	string dpe = sDp+".alarm";
	bool isActive;
	dyn_string dsAlertText, dsAlertClass;
	
//DebugN("_unSystemIntegrity_createSystemAlarm", sDp);
	dpCreate(sDp, c_unSystemAlarm_dpType);
// a second check is done after the creation to ensure that the data point is created (just in case of error)
// it is also a way to wait the creation which is asynchronous.
	if(!dpExists(sDp)) {
		fwException_raise(exceptionInfo, "ERROR", 
						"_unSystemIntegrity_createSystemAlarm():"+getCatStr("unSystemIntegrity", "CANNOT_CREATE_DP")+sDp,"");
	}
	else {
// set its alarm config
		if(bOkRange) {
			dsAlertText = makeDynString(UN_SYSTEMINTEGRITY_OK, UN_SYSTEMINTEGRITY_BAD);
			dsAlertClass = makeDynString("", getSystemName()+"_unSystemIntegrityAlarm_lvl1.");
		}
		else {
			dsAlertText = makeDynString(UN_SYSTEMINTEGRITY_BAD, UN_SYSTEMINTEGRITY_OK);
			dsAlertClass = makeDynString(getSystemName()+"_unSystemIntegrityAlarm_lvl1.", "");
		}
		fwAlertConfig_setGeneral(dpe, dsAlertText, 
									makeDynFloat(c_unSystemIntegrity_alarm_value_level1), 
									dsAlertClass, 
									"", makeDynString(), "", 1, exceptionInfo);
// set the description
		dpSetDescription(dpe, description);
	}
}

// _unSystemIntegrity_createScheduler
/**
Purpose:
This creates the dp and sets the schedulerwith default settings.

	@param sDp: string, input, data point name = Scheduler name
	@param iEr: int, output, Error code
        @param sSystemName: string, input, the system name, default =""=getSystemName()

Usage: Internal

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/
_unSystemIntegrity_createScheduler(string sCat, int &iEr, string sSystemName="")
{
  string sDp;
  int iNum;
  
//DebugN("_unSystemIntegrity_createScheduler", sCat, sSystemName);
  if(sSystemName == "")
    sSystemName = getSystemName();
  if(sSystemName == getSystemName())
    iEr=dpCreate(sCat,"_Sc");
  else
  {
    iNum = getSystemId(sSystemName);
    if(iNum >= 0)
      iEr=dpCreate(sCat,"_Sc", iNum);
//DebugN(iNum, sSystemName, sCat, iEr);
  }
  sDp = sSystemName+sCat;
//DebugN(sDp, dpExists(sDp));
  if(!dpExists(sDp))
    iEr = -1;
if (iEr>=0)
	{
	iEr=dpSetWait(	sDp + ".name", sCat,
					sDp + ".type", "F",
					sDp + ".mode", false,
					sDp + ".logEntry", true,
					sDp + ".actions.dpeList.dpes", makeDynString("_unSendMail.message.periodic_report"),	//dpe to set on action
					sDp + ".actions.dpeList.dpeValues", makeDynString(sCat),									//set Type
					sDp + ".actions.dpeList.active", true,									
					sDp + ".time.timedFunc.validUntil", makeTime(2034,12,31,23,59,59),						//Max
					sDp + ".time.timedFunc.interval",86400,													//24 hours
					sDp + ".time.timedFunc.syncTime",0,														//01/01/this year at 0:00
					sDp + ".time.timedFunc.syncWeekDay",-1,
					sDp + ".time.timedFunc.syncDay",1,
					sDp + ".time.timedFunc.syncMonth",1,		
					sDp + ".time.conditionType",1);															//Periodic
	
        if(sSystemName == getSystemName())
	  iEr=dpSetDescription(sDp + ".name", sCat);  
	}
}

// _unSystemIntegrity_setList
/**
Purpose:
Add/remove a string in a list. Indivisible function

	@param dsList: dyn_string, input & output, list to look in
	@param sDp: string, input, string to find in the list
	@param toAdd: bool, input, true add in list if not already added, false remove from the list if sDp is in the list

Usage: Internal

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/
int _unSystemIntegrity_setList(dyn_string &dsList, string sDp, bool bAdd)
{
	int pos;
	
	pos = dynContains(dsList, sDp);
	if(bAdd) { //add in list
		if(pos <= 0) {// if not in list do nothing
			dynAppend(dsList, sDp);
			pos = dynlen(dsList);
		}
	}
	else { // remove from list
		if(pos > 0) // if in list remove it
			dynRemove(dsList, pos);
	}
//DebugN("_unSystemIntegrity_setList",dsList, sDp, bAdd);
	return pos;
}

// _unSystemIntegrity_isInList
/**
Purpose:
Add/remove a string in a list. Indivisible function

	@param dsList: dyn_string, input & output, list to look in
	@param sDp: string, input, string to find in the list

Usage: Internal

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/
int _unSystemIntegrity_isInList(dyn_string &dsList, string sDp)
{
	int pos;
	
	pos = dynContains(dsList, sDp);
//DebugN("_unSystemIntegrity_isInList",dsList, pos);
	return pos;
}

// _unSystemIntegrity_registerFunction
/**
Purpose:
Register function, does the dpConnect and dpDisconnect, add/remove the dp in/from the global list.

	@param bRegister: bool, input, true: add, false: remove from sGlobalVar
	@param sCallBack: string, input, callback function
	@param dsCallBackParam: dyn_string, input, callback parameters
	@param sGlobalVar: dyn_string, input-output, the global variable used to store the list of dp to check
	@param sDp: string, input, the dp to add/remove in/from the sGlobalVar
	@param exceptionInfo: dyn_string, output, the error is returned here

Usage: Internal

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/
_unSystemIntegrity_registerFunction(bool bRegister, string sCallBack, dyn_string dsCallBackParam, 
					dyn_string &sGlobalVar, string sDp, dyn_string &exceptionInfo)
{
	int res;
	
	if(bRegister) {
// connect to the callback function if not already done
		res = _unSystemIntegrity_isInList(sGlobalVar, sDp);
		if(res <= 0) {
		// add in list first because if the dpConnect succeeds the callback may be called before the list is updated
		// and in the callback the list is used
			res = _unSystemIntegrity_setList(sGlobalVar, sDp, true);

//DebugN("gl",sGlobalVar);
 			
 			switch(dynlen(dsCallBackParam)) {
				case 1:
					res = dpConnect(sCallBack, dsCallBackParam[1]);
					break;
				case 2:
					res = dpConnect(sCallBack, dsCallBackParam[1], dsCallBackParam[2]);
					break;
				case 3:
					res = dpConnect(sCallBack, dsCallBackParam[1], dsCallBackParam[2],dsCallBackParam[3]);
					break;
				default:
					res = -1;
					break;
			}
			if(res < 0) {
				fwException_raise(exceptionInfo, "ERROR", 
							"_unSystemIntegrity_registerFunction(): "+getCatStr("unSystemIntegrity", "CANNOT_DP_CONNECT") + dsCallBackParam[1],"");
			
			// remove from list in case of error
				res = _unSystemIntegrity_setList(sGlobalVar, sDp, false);
			}
			else {
		// add in list
				res = _unSystemIntegrity_setList(sGlobalVar, sDp, true);
			// set the enable to true and activate the alarm if it is not activated.
				dpSet(sDp+".enabled", true);
				unAlarmConfig_mask(sDp+".alarm", false, exceptionInfo);
			}
		}
	}
	else {
// disconnect the callback function
		res = _unSystemIntegrity_isInList(sGlobalVar, sDp);
		if(res > 0) {
			switch(dynlen(dsCallBackParam)) {
				case 1:
					res = dpDisconnect(sCallBack, dsCallBackParam[1]);
					break;
				case 2:
					res = dpDisconnect(sCallBack, dsCallBackParam[1], dsCallBackParam[2]);
					break;
				case 3:
					res = dpDisconnect(sCallBack, dsCallBackParam[1], dsCallBackParam[2],dsCallBackParam[3]);
					break;
				default:
					res = -1;
					break;
			}
			if(res < 0) {
				fwException_raise(exceptionInfo, "ERROR", 
							"_unSystemIntegrity_registerFunction():"+getCatStr("unSystemIntegrity", "CANNOT_DP_DISCONNECT") + sDp,"");
			}
			else {
			// remove from list
				res = _unSystemIntegrity_setList(sGlobalVar, sDp, false);
			// set the enable to false and de-activate the alarm and acknowledge it if necessary
				dpSet(sDp+".enabled", false, sDp+".alarm", c_unSystemIntegrity_no_alarm_value);
				unAlarmConfig_mask(sDp+".alarm", true, exceptionInfo);
			}
		}
	}
//DebugN("end",sGlobalVar);

}

// unSystemIntegrity_getComponents
/**
Purpose:
get the list of components for the systemIntegrity

	@param dsComponents: dyn_string, output, the list of components is returned here

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/
unSystemIntegrity_getComponents(dyn_string &dsComponents)
{
	dyn_string dsTemp;
	int i, length;
	string sSystemName;
	
	sSystemName = getSystemName();
	dsTemp = dpNames("*" + UN_SYSTEMINTEGRITY_EXTENSION, UN_SYSTEMINTEGRITY_DPTYPE);	// Get DPEs
//DebugN(dsTemp, UN_SYSTEMINTEGRITY_EXTENSION, UN_SYSTEMINTEGRITY_DPTYPE);
	length = dynlen(dsTemp);
	for(i=1;i<=length;i++)					// For each element, delete system name
	{
		dsTemp[i] = substr(dsTemp[i], strlen(sSystemName));
		dsTemp[i] = substr(dsTemp[i], 0, strlen(dsTemp[i]) - strlen(UN_SYSTEMINTEGRITY_EXTENSION));
	}
	dynUnique(dsTemp);
	dynSortAsc(dsTemp);
//DebugN(dsTemp, UN_SYSTEMINTEGRITY_EXTENSION, UN_SYSTEMINTEGRITY_DPTYPE);
	dsComponents = dsTemp;
}

// unSystemIntegrity_getComponentsFromSystem
/**
Purpose:
get the list of components for the systemIntegrity

	@param dsComponents: dyn_string, output, the list of components is returned here

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/
unSystemIntegrity_getComponentsFromSystem(dyn_string &dsComponents, string sSystemName)
{
	dyn_string dsTemp;
	int i, length;
	
	dsTemp = dpNames(sSystemName+"*" + UN_SYSTEMINTEGRITY_EXTENSION, UN_SYSTEMINTEGRITY_DPTYPE);	// Get DPEs
//DebugN(dsTemp, UN_SYSTEMINTEGRITY_EXTENSION, UN_SYSTEMINTEGRITY_DPTYPE);
	length = dynlen(dsTemp);
	for(i=1;i<=length;i++)					// For each element, delete system name
	{
		dsTemp[i] = substr(dsTemp[i], strlen(sSystemName));
		dsTemp[i] = substr(dsTemp[i], 0, strlen(dsTemp[i]) - strlen(UN_SYSTEMINTEGRITY_EXTENSION));
	}
	dynUnique(dsTemp);
	dynSortAsc(dsTemp);
//DebugN(dsTemp, UN_SYSTEMINTEGRITY_EXTENSION, UN_SYSTEMINTEGRITY_DPTYPE);
	dsComponents = dsTemp;
}

// unSystemIntegrity_getComponentsFunctions
/**
Purpose:
get the list function of a given component for the systemIntegrity

	@param sComponent: string, input, the component,
	@param dsFunctions: dyn_string, output, the list of function is returned here

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/
unSystemIntegrity_getComponentsFunctions(string sComponent, dyn_string &dsFunctions, string sSystem=getSystemName())
{
	dpGet(sSystem + sComponent + UN_SYSTEMINTEGRITY_EXTENSION + ".config.functions", dsFunctions);
}

// unSystemIntegrity_getComponentsPanels
/**
Purpose:
get the list function of a given component for the systemIntegrity

	@param sComponent: string, input, the component,
	@param dsPanels: dyn_string, output, the panels are returned here

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/
unSystemIntegrity_getComponentsPanels(string sComponent, dyn_string &dsPanels, string sSystem=getSystemName())
{
	dpGet(sSystem + sComponent + UN_SYSTEMINTEGRITY_EXTENSION + ".config.configurationPanel", dsPanels[1],
			sSystem + sComponent + UN_SYSTEMINTEGRITY_EXTENSION + ".config.operationPanel", dsPanels[2]);
}

// unSystemIntegrity_getComponentsData
/**
Purpose:
get the data of a given component for the systemIntegrity

	@param sComponent: string, input, the component,
	@param dsData: dyn_string, output, the config data are returned here

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/
unSystemIntegrity_getComponentsData(string sComponent, dyn_string &dsData, string sSystem=getSystemName())
{
	dpGet(sSystem + sComponent + UN_SYSTEMINTEGRITY_EXTENSION + ".config.data", dsData);
}

// unSystemIntergity_manageManager
/**
Purpose:
Stop, kill, start a manager corresponding to sManager and sCommandLine

	@param sManager: string, input, the PVSS manager,
	@param sCommandLine: string, input, the command line of PVSS manager,
	@param sStopCommand: string, input, the command: SYSTEMINTEGRITY_STOP_COMMAND, SYSTEMINTEGRITY_KILL_COMMAND, SYSTEMINTEGRITY_START_COMMAND, SYSTEMINTEGRITY_DEL_COMMAND
	@param iManagerIndex: int, output, the index of the PVSS manager,
	@param exceptionInfo: dyn_string, output, the errors are returned here

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/
unSystemIntergity_manageManager(string sManager, string sCommandLine, string sStopCommand, int &iManagerIndex, dyn_string &exceptionInfo, string sSystem="")
{
int iPort;
string sHost, sIP;

	if (sSystem=="")	
		sSystem=getSystemName();
	
	unGenericDpFunctions_getHostName(sSystem, sHost, sIP);	
	unGenericDpFunctions_getpmonPort(sSystem, iPort);
		
	if (sHost=="" || iPort==0)
		{
		sHost=getHostname();
		iPort=pmonPort();
		}
		
	if (sHost!="" && iPort>0)		
		unSystemIntergity_manageManagerperHostname(sManager, sCommandLine, sStopCommand, sHost, iPort, iManagerIndex, exceptionInfo);
	else
		fwException_raise(exceptionInfo, "ERROR", "unSystemIntergity_manageManager() - Unable to get the Hostname or PMON port number!", "");		
}

// unSystemIntergity_manageManagerperHostname
/**
Purpose:
Stop, kill, start a manager corresponding to sManager and sCommandLine

	@param sManager: string, input, the PVSS manager,
	@param sCommandLine: string, input, the command line of PVSS manager,
	@param sStopCommand: string, input, the command: SYSTEMINTEGRITY_STOP_COMMAND, SYSTEMINTEGRITY_KILL_COMMAND, SYSTEMINTEGRITY_START_COMMAND, SYSTEMINTEGRITY_DEL_COMMAND
	@param sHost: string, input, computer hostname,
	@param iPort: int, input, pMon port number,	
	@param iManagerIndex: int, output, the index of the PVSS manager,
	@param exceptionInfo: dyn_string, output, the errors are returned here

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.0 
	. operating system: Linux, Windows XP
	. distributed system: yes
*/
unSystemIntergity_manageManagerperHostname(string sManager, string sCommandLine, string sStopCommand, string sHost, int iPort, int &iManagerIndex, dyn_string &exceptionInfo)
{
bool bFailed;
dyn_dyn_string ddsResult;
int i, iLen, iManagerState;
int iIndex=SYSTEMINTEGRITY_PMON_UNKNOWN_VALUE;
string sPmonCommand;
	
	switch(sStopCommand)
		{
		case SYSTEMINTEGRITY_STOP_COMMAND:
			sPmonCommand=SYSTEMINTEGRITY_PMON_STOP_COMMAND;
			break;
		case SYSTEMINTEGRITY_KILL_COMMAND:
			sPmonCommand=SYSTEMINTEGRITY_PMON_KILL_COMMAND;
			break;
		case SYSTEMINTEGRITY_START_COMMAND:
			sPmonCommand=SYSTEMINTEGRITY_PMON_START_COMMAND;
			break;
		case SYSTEMINTEGRITY_DEL_COMMAND:
			sPmonCommand=SYSTEMINTEGRITY_PMON_DEL_COMMAND;
			break;
		case SYSTEMINTEGRITY_SETMANUAL_COMMAND:
		case SYSTEMINTEGRITY_SETALWAYS_COMMAND:
			sPmonCommand=SYSTEMINTEGRITY_PMON_SETMANUAL_COMMAND;
			break;
		default:
			break;
		}
	
	// get the list of manager
	bFailed = pmon_query(USERNAME+"#"+PASSWORD+"#MGRLIST:LIST", sHost, iPort, ddsResult, false, true);
	if(!bFailed)
		{			
		iLen = dynlen(ddsResult);
		for(i=1;i<=iLen;i++)
			{
                  if(dynlen(ddsResult[i]) == 5) // case no argument
                    dynAppend(ddsResult[i], "");
			if (dynlen(ddsResult[i])>=6)
				{					
				// if the manager is found with the command line - " " is needed to distinguish -num 1 and -num 10
				if((sManager == ddsResult[i][1]) && (strpos(ddsResult[i][6] + " ",sCommandLine + " ") >=0))
					{
					iIndex=i-1;
					if(sPmonCommand != "")
						{
						sPmonCommand = USERNAME + "#" + PASSWORD + "#" + sPmonCommand + iIndex;
					
						// get the state of the manager
						unSystemIntergity_getManagerStateperHostname(iIndex, sHost, iPort, iManagerState, exceptionInfo);
						switch(sStopCommand)
							{
							case SYSTEMINTEGRITY_START_COMMAND:
							case SYSTEMINTEGRITY_DEL_COMMAND:
								if(iManagerState == SYSTEMINTEGRITY_MANAGER_STOPPED)
									bFailed = pmon_command(sPmonCommand, sHost, iPort, FALSE, TRUE);
								else 
									bFailed = true;
								break;
							case SYSTEMINTEGRITY_SETMANUAL_COMMAND:
								sPmonCommand = sPmonCommand+ 
											" " + SYSTEMINTEGRITY_MANAGER_MANUAL +	//Mode
											" " + ddsResult[i][3] +															//Seconds to kill
											" " + ddsResult[i][4] +																//Restart
											" " + ddsResult[i][5] +																//Reset Start counter
											" " + ddsResult[i][6];									//Command Line
								bFailed = pmon_command(sPmonCommand, sHost, iPort, FALSE, TRUE);
								break;
							case SYSTEMINTEGRITY_SETALWAYS_COMMAND:
								sPmonCommand = sPmonCommand+ 
											" " + SYSTEMINTEGRITY_MANAGER_ALWAYS +	//Mode
											" " + ddsResult[i][3] +															//Seconds to kill
											" " + ddsResult[i][4] +																//Restart
											" " + ddsResult[i][5] +																//Reset Start counter
											" " + ddsResult[i][6];									//Command Line
								bFailed = pmon_command(sPmonCommand, sHost, iPort, FALSE, TRUE);
								break;								
							default:
								if((iManagerState == SYSTEMINTEGRITY_MANAGER_RUNNING) || (iManagerState == SYSTEMINTEGRITY_MANAGER_BLOCKED))
									bFailed = pmon_command(sPmonCommand, sHost, iPort, FALSE, TRUE);
								else if(myManType() == UI_MAN)
            {
  								if((iManagerState == SYSTEMINTEGRITY_MANAGER_RUNNING) || (iManagerState == SYSTEMINTEGRITY_MANAGER_BLOCKED)
                    || (iManagerState == SYSTEMINTEGRITY_MANAGER_INIT))
  									bFailed = pmon_command(sPmonCommand, sHost, iPort, FALSE, TRUE);
            }
            else
									bFailed = true;
								break;
							}
						if(bFailed)
							fwException_raise(exceptionInfo, "ERROR", "unSystemIntergity_manageManagerperHostname() - " + getCatStr("unSystemIntegrity", "CANNOT_EXECUTE_PMON_COMMAND") + sPmonCommand, "");	
						}//end: sPmonCommand != ""
					i = iLen+1;	// exit the loop;	
					}// end: if the manager is found with the command line
				}// end: dynlen(ddsResult[i])>=6							
			}//end: for
		}
	else
		fwException_raise(exceptionInfo, "ERROR", "unSystemIntergity_manageManagerperHostname() - " + getCatStr("unSystemIntegrity", "CANNOT_CONNECT_PMON"), "");
	
	iManagerIndex = iIndex;
//	if(iManagerIndex==SYSTEMINTEGRITY_PMON_UNKNOWN_VALUE)
//		DebugN("unSystemIntergity_manageManagerperHostname() - Manager not defined: " + sManager + " " + sCommandLine, sHost, iPort);
}

// unSystemIntergity_getManagerState
/**
Purpose:
Stop, kill, start a manager corresponding to sManager and sCommandLine

	@param iManagerIndex: int, input, the index of the PVSS manager returned by unSystemIntergity_manageManager
	@param iManagerState: int, output, the state of the PVSS manager:
	@param exceptionInfo: dyn_string, output, the errors are returned here
	@param dsResult: dyn_string, output, Full ManagerState

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/
dyn_string unSystemIntergity_getManagerState(int iManagerIndex, int &iManagerState, dyn_string &exceptionInfo, string sSystem="")
{
int iPort;
string sHost, sIP;
dyn_string dsResult;
	
	if (sSystem=="")
		sSystem=getSystemName();
	
	unGenericDpFunctions_getHostName(sSystem, sHost, sIP);	
	unGenericDpFunctions_getpmonPort(sSystem, iPort);
	
	if (sHost=="" || iPort==0)
		{
		sHost=getHostname();
		iPort=pmonPort();
		}
		
	if (sHost!="" && iPort>0)
		dsResult=unSystemIntergity_getManagerStateperHostname(iManagerIndex, sHost, iPort, iManagerState, exceptionInfo);
	else
		fwException_raise(exceptionInfo, "ERROR", "unSystemIntergity_getManagerState() - Unable to get the Hostname or PMON port number!", "");
		
	return dsResult;	
}

// unSystemIntergity_getManagerStateperHostname
/**
Purpose:
Stop, kill, start a manager corresponding to sManager and sCommandLine on a Local or Remote system

	@param iManagerIndex: int, input, the index of the PVSS manager returned by unSystemIntergity_manageManager
	@param sHost: string, input, Hostname
	@param iPort: int, input, Port number of the Host
	@param iManagerState: int, output, the state of the PVSS manager:
	@param exceptionInfo: dyn_string, output, the errors are returned here
	@param dsResult: dyn_string, output, Full ManagerState

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/

dyn_string unSystemIntergity_getManagerStateperHostname(int iManagerIndex, string sHost, int iPort, int &iManagerState, dyn_string &exceptionInfo)
{
dyn_dyn_string ddsResult;
dyn_string dsResult;
bool bFailed;
int iState = SYSTEMINTEGRITY_PMON_UNKNOWN_VALUE;
	
	if(iManagerIndex>SYSTEMINTEGRITY_PMON_UNKNOWN_VALUE)
		{
		bFailed = pmon_query (USERNAME+"#"+PASSWORD+"#"+SYSTEMINTEGRITY_PMON_QUERY_COMMAND, sHost, iPort, ddsResult, FALSE, TRUE);
		if((!bFailed) && (dynlen(ddsResult) > 0))
			{
			if(iManagerIndex < dynlen(ddsResult))
				{
				iState = ddsResult[iManagerIndex+1][1];
				dsResult = ddsResult[iManagerIndex+1];
				}
			}
		else
			fwException_raise(exceptionInfo, "ERROR", "unSystemIntergity_getManagerStateperHostname() - " + getCatStr("unSystemIntegrity", "CANNOT_CONNECT_PMON"), "");
		}
	else
		fwException_raise(exceptionInfo, "ERROR", "unSystemIntergity_getManagerStateperHostname() - Manager not defined", "");	
	
	iManagerState = iState;
	return dsResult;
}

//------------------------------------------------------------------------------------------------------------------------
// unSystemIntegrity_getState
/**
Purpose:
get the current state of a manager

	@param sManager: string, input, manager name
	@param sCommandLine: int, input, the manager command line
	@param iManagerState: int, output, the manager state

Usage: Internal

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/
unSystemIntegrity_getState(string sManager, string sCommandLine, int &iManagerState)
{
	bool bFailed;
	dyn_dyn_string ddsResult;
	int i, iLen;
	int iIndex=SYSTEMINTEGRITY_PMON_UNKNOWN_VALUE;
	int iPort;
	string sHost, sIP,sSystem;
	dyn_string exceptionInfo;
	
	iManagerState = SYSTEMINTEGRITY_PMON_UNKNOWN_VALUE;

	if (sSystem=="")
		sSystem=getSystemName();
	
	unGenericDpFunctions_getHostName(sSystem, sHost, sIP);	
	unGenericDpFunctions_getpmonPort(sSystem, iPort);
	
	if (sHost=="" || iPort==0)
		{
		sHost=getHostname();
		iPort=pmonPort();
		}
		
	if (sHost!="" && iPort>0)
	{
		// get the list of manager
		bFailed = pmon_query(USERNAME+"#"+PASSWORD+"#MGRLIST:LIST", sHost, iPort, ddsResult, false, true);
		if(!bFailed)
		{			
			iLen = dynlen(ddsResult);
			for(i=1;i<=iLen;i++)
			{
				if (dynlen(ddsResult[i])>=6)
				{					
					// if the manager is found with the command line - " " is needed to distinguish -num 1 and -num 10
					if((sManager == ddsResult[i][1]) && (strpos(ddsResult[i][6] + " ",sCommandLine + " ") >=0))
					{
						iIndex=i-1;
						// get the state of the manager
						unSystemIntergity_getManagerStateperHostname(iIndex, sHost, iPort, iManagerState, exceptionInfo);
						i = iLen+1;	// exit the loop;	
					}// end: if the manager is found with the command line
				}// end: dynlen(ddsResult[i])>=6							
			}//end: for
		}
	}
}

// _unSystemIntegrity_createScriptScheduler
/**
Purpose:
This function creates the dp and sets the scheduler with default settings.

	@param sDp: string, input, data point name = Scheduler name
	@param iEr: int, output, Error code
        @param sSystemName: string, input, the system name, default =""=getSystemName()

Usage: Internal

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/
_unSystemIntegrity_createScriptScheduler(string sCat, int &iEr, string sScript, int iInterval, string sSystemName="")
{
  string sDp;
  int iNum;
  
//DebugN("_unSystemIntegrity_createScheduler", sCat, sSystemName);
  if(sSystemName == "")
    sSystemName = getSystemName();
  if(sSystemName == getSystemName())
    iEr=dpCreate(sCat,"_Sc");
  else
  {
    iNum = getSystemId(sSystemName);
    if(iNum >= 0)
      iEr=dpCreate(sCat,"_Sc", iNum);
//DebugN(iNum, sSystemName, sCat, iEr);
  }
  sDp = sSystemName+sCat;
//DebugN(sDp, dpExists(sDp));
  if(!dpExists(sDp))
    iEr = -1;
  if (iEr>=0)
  {
    dpSetWait(sDp + ".name", sCat,
                  sDp + ".type", "F",
                  sDp + ".mode", true,
                  sDp + ".logEntry", true,
                  sDp + ".actions.script.path", sScript,
                  sDp + ".actions.script.active", true,
                  sDp + ".time.timedFunc.validUntil", makeTime(2034,12,31,23,59,59),
                  sDp + ".time.timedFunc.interval",iInterval,	
                  sDp + ".time.timedFunc.syncTime",0,
                  sDp + ".time.timedFunc.syncWeekDay",-1,
                  sDp + ".time.timedFunc.syncDay",1,
                  sDp + ".time.timedFunc.syncMonth",1,
                  sDp + ".time.conditionType",1);															//Periodic
	
    if(sSystemName == getSystemName())
      dpSetDescription(sDp + ".name", sCat); 
    dpSet(sDp + ".actions.reminderAlarm.alarmS:_alert_hdl.._type", DPCONFIG_ALERT_BINARYSIGNAL,
          sDp + ".actions.reminderAlarm.alarmS:_alert_hdl.._ok_range", 0,
          sDp + ".actions.reminderAlarm.alarmS:_alert_hdl.._class", "scAlarm.",
          sDp + ".actions.reminderAlarm.alarmS:_alert_hdl.._orig_hdl", 1,
          sDp + ".actions.reminderAlarm.alarmD:_alert_hdl.._type", DPCONFIG_ALERT_NONBINARYSIGNAL,
          sDp + ".actions.reminderAlarm.alarmD:_alert_hdl.1._type", 4,
          sDp + ".actions.reminderAlarm.alarmD:_alert_hdl.2._type", 4,
          sDp + ".actions.reminderAlarm.alarmD:_alert_hdl.3._type", 4,
          sDp + ".actions.reminderAlarm.alarmD:_alert_hdl.1._u_limit", 0,
          sDp + ".actions.reminderAlarm.alarmD:_alert_hdl.2._u_limit", 1,
          sDp + ".actions.reminderAlarm.alarmD:_alert_hdl.2._l_limit", 0,
          sDp + ".actions.reminderAlarm.alarmD:_alert_hdl.3._l_limit", 1,
          sDp + ".actions.reminderAlarm.alarmD:_alert_hdl.1._u_incl", FALSE,
          sDp + ".actions.reminderAlarm.alarmD:_alert_hdl.2._u_incl", FALSE,
          sDp + ".actions.reminderAlarm.alarmD:_alert_hdl.3._u_incl", TRUE,
          sDp + ".actions.reminderAlarm.alarmD:_alert_hdl.1._l_incl", TRUE,
          sDp + ".actions.reminderAlarm.alarmD:_alert_hdl.2._l_incl", TRUE,
          sDp + ".actions.reminderAlarm.alarmD:_alert_hdl.3._l_incl", TRUE,
          sDp + ".actions.reminderAlarm.alarmD:_alert_hdl.1._class", "scAlarm.",
          sDp + ".actions.reminderAlarm.alarmD:_alert_hdl.3._class", "scAlarm.",
          sDp + ".actions.reminderAlarm.alarmD:_alert_hdl.._orig_hdl", 1);
  }
}

// _unSystemIntegrity_startScheduler
/**
Purpose:
This function adds the scheduler.

	@param sDp: string, input, data point name = Scheduler name
	@param iEr: int, output, Error code
        @param sSystemName: string, input, the system name, default =""=getSystemName()

Usage: Internal

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/
_unSystemIntegrity_startScheduler(string sCat, string sDp, string sSystemName="")
{
  dpSetWait(sSystemName+"_ScCom.transfer:_original.._value", sCat +"|"+ dpSubStr(sSystemName+sDp, DPSUB_SYS_DP)); 
}

//------------------------------------------------------------------------------------------------------------------------
// _unSystemIntegrity_createSystemAlarmBasic
/** This function creates the systemAlarm dp and sets the alert hdl with more than one alert level if needed.
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDp  input, data point name
@param description input, device type
@param dsAlertText input, list of alert text
@param dfLimits input, list of alert limits
@param dsAlertClass input, list of alert Class
@param exceptionInfo output, exception are returned here
*/
_unSystemIntegrity_createSystemAlarmBasic(string sDp, string description, dyn_string dsAlertText, dyn_float dfLimits, dyn_string dsAlertClass, dyn_string &exceptionInfo)
{
  string dpe = sDp+".alarm";
  bool isActive;
	
  //DebugN("_unSystemIntegrity_createSystemAlarm", sDp);
  dpCreate(sDp, c_unSystemAlarm_dpType);
  // a second check is done after the creation to ensure that the data point is created (just in case of error)
  // it is also a way to wait the creation which is asynchronous.
  if(!dpExists(sDp)) 
  {
    fwException_raise(exceptionInfo, "ERROR", 
                      "_unSystemIntegrity_createSystemAlarm():"+getCatStr("unSystemIntegrity", "CANNOT_CREATE_DP")+sDp,"");
  }
  else 
  {
    // set its alarm config
    fwAlertConfig_setGeneral(dpe, dsAlertText, dfLimits, dsAlertClass,
                             "", makeDynString(), "", 1, exceptionInfo); 
    // set the description
    dpSetDescription(dpe, description);
  }
}

//------------------------------------------------------------------------------------------------------------------------
// unSystemIntegrity_addManager
/** This function allows to insert a manager into a project. It is checked before, if the 
manager already exists.
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL



@param manager				name of the manager
@param startMode			{manual, once, always}
@param secKill				seconds to kill after stop
@param restartCount		number of restarts
@param resetMin				restart counter reset time (minutes)
@param commandLine		commandline for the manager
@param exceptionInfo	errors are returned here
@param sSystem	the system name, "" = getSystemName()
@return 1 - manager added, 2 - manager already existing, 3 - manager addition disabled, 0 - manager addition failed
*/
int unSystemIntegrity_addManager(string manager, 
                              string startMode, 
                              int secKill, 
                              int restartCount, 
                              int resetMin,
                              string commandLine, 
                              dyn_string &exceptionInfo,
                              string sSystem ="")
{
  bool failed, disabled;
  string str;
  dyn_mixed managerInfo;
  string sHost, sIP;
  string user, pwd;
  int iPort;
  
  if (sSystem=="")	
    sSystem=getSystemName();
	
  unGenericDpFunctions_getHostName(sSystem, sHost, sIP);	
  unGenericDpFunctions_getpmonPort(sSystem, iPort);
  
  if (sHost=="" || iPort==0)
  {
		sHost=getHostname();
		iPort=pmonPort();
  }
  
  if(fwInstallation_getPmonInfo(user, pwd) != 0)
  {
    fwException_raise(exceptionInfo, "ERROR", "unSystemIntegrity_addManager() - Unable to get the user and passwd!", "");		
    return -1;
  }
  
  fwInstallationManager_getProperties(manager, commandLine, managerInfo, sHost, iPort, user, pwd);
  if(managerInfo[FW_INSTALLATION_MANAGER_PMON_IDX] != -1)
  {
    return 2; //Manager already in the PVSS console
  }
  str = user + "#" + pwd + "#SINGLE_MGR:INS "+unSystemIntegrity_pmonGetCount(sHost, iPort)+" "+
//  str = user + "#" + pwd + "#SINGLE_MGR:INS "+pmonGetCount()+" "+
        manager+" "+startMode+" "+secKill+" "+restartCount+" "+
        resetMin+" "+commandLine;
  
  if(pmon_command(str, sHost, iPort, FALSE, TRUE))
  {
    fwException_raise(exceptionInfo, "ERROR", "unSystemIntegrity_addManager(): Failed to insert manager: "  + manager + " " + commandLine + " " + sHost + " " + iPort + " "+user, "");		
    return 0;
  } 
//DebugTN(str, sHost, iPort);
  return 1;
}

//------------------------------------------------------------------------------------------------------------------------
// unSystemIntegrity_pmonGetCount
/** Equivalent to pmonGetCount(). This function returns the number of manager in the console.
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sHost  input, pmon hostname
@param iPort input, pmon port number
@return the number of manager in the console
*/
int unSystemIntegrity_pmonGetCount(string sHost, int iPort)
{
  bool bFailed;
  int iPmonCount = SYSTEMINTEGRITY_PMON_UNKNOWN_VALUE, i, len;
  dyn_dyn_string ddsResult;
  
  bFailed = pmon_query(USERNAME+"#"+PASSWORD+"#MGRLIST:LIST", sHost, iPort, ddsResult, false, true);
  if(!bFailed)
  {			
    len = dynlen(ddsResult);
    for(i=1;i<=len;i++)
    {
      if(dynlen(ddsResult[i]) == 5) // case no argument
        dynAppend(ddsResult[i], "");
      if (dynlen(ddsResult[i])>=6)
      {			
        iPmonCount = i;
      }
    }
  }
//  DebugTN(pmonGetCount(), iPmonCount);
  return iPmonCount;
}

//------------------------------------------------------------------------------------------------------------------------

//@}


