/**@name LIBRARY: unicosHMI.ctl

@author: Herve Milcent (AB-CO)

Creation Date: 24/10/2008

Modification History: 
  - 11/11/2011: Herve
  IS-606  unCore - unGraphicalFrame  HMI interface: set system state to unknown for newly added remote system 
 
version 1.0

External Functions:
  - unicosHMI_init: HMI init function
  - unicosHMI_applicationUpdateDeviceList: HMI device data  trigger update function
  - unicosHMI_registerTreeDeviceOverview: HMI register function
	
Internal Functions: 
  - _unicosHMI_TreeDeviceOverview_register: Distributed control register callback function
  
Purpose: 
This library contain generic functions to be used with the unicosHMI.

Usage: Public

PVSS manager usage: UI (PVSS00ui) 

Constraints:
  . PVSS version: 3.6 
  . operating system: WXP & Linux.
  . distributed system: yes.
*/

//@{
//---------------------------------------------------------------------------------------------------------------------------------------

// unicosHMI_init
/**
Purpose:
HMI init function. Put here any initialization, global, etc. This function is executed by evalScript

  @param return value: bool, output, true=OK/false=BAD

Usage: Public

PVSS manager usage: UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
  . function executed by evalScript->no dpConnect, dpQueryConnect, startThread, etc.
  . PVSS version: 3.6 
  . operating system: WXP and Linux.
  . distributed system: yes.
*/
bool unicosHMI_init()
{
   // initialise the number of max opened module
//  DebugTN("unicosHMI_init");
  g_iMaxOpenedModule = MAX_OPENED_MODULE;
  return unGraphicalFrame_checkLibLoaded(makeDynString());
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unicosHMI_applicationUpdateDeviceList
/**
Purpose:
HMI device data  trigger update function. Callback function triggered when the device data is updated 
Put here any initialization, global, etc. This function is executed by dpConnect

  @param return value: bool, output, true=OK/false=BAD

Usage: Public

PVSS manager usage: UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
  . PVSS version: 3.6 
  . operating system: WXP and Linux.
  . distributed system: yes.
*/
unicosHMI_applicationUpdateDeviceList(string sdpe, int iCommand, string sdpe2, string sSystemName)
{
//DebugTN("unicosHMI_applicationUpdateDeviceList", iCommand, sSystemName);
  synchronized(g_dsTreeApplication) {
    if(mappingHasKey(g_m_bTreeDeviceOverviewRegisteredSystemInitialized, sSystemName))
    {
      unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "TreeGlobal", "unicosHMI_applicationUpdateDeviceList", sSystemName+" updated");
      g_m_unicosGraphicalFrame_bUpdate[sSystemName] = true;
      if(g_m_bTreeDeviceOverviewRegisteredSystemInitialized[sSystemName]) {
       if(!g_m_bTreeDeviceOverview_gettingDeviceRunning[sSystemName]) {
    //DebugTN("trigger upda", sSystemName);
         g_m_bTreeDeviceOverview_gettingDeviceRunning[sSystemName] = true;
         unTreeDeviceOverview_loadAll(sSystemName, g_dsDeviceType);
         unGraphicalFrame_triggerDisplay(DEVUN_ADD_COMMAND, sSystemName);
         g_m_bTreeDeviceOverview_gettingDeviceRunning[sSystemName] = false;
       }
      }
    }
    else
      unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "TreeGlobal", "unicosHMI_applicationUpdateDeviceList", sSystemName+" not defined");
  }
//DebugTN("end unicosHMI_applicationUpdateDeviceList", iCommand, sSystemName);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unicosHMI_registerTreeDeviceOverview
/**
Purpose:
HMI register function. Put here any code to be executed once at startup, etc. This function is executed by startThread

  @param return value: bool, output, true=OK/false=BAD

Usage: Public

PVSS manager usage: UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
  . PVSS version: 3.6 
  . operating system: WXP and Linux.
  . distributed system: yes.
*/
bool unicosHMI_registerTreeDeviceOverview()
{
  int i, len;
  dyn_string dsSystemNames, dsHost, exInfo;
  dyn_int diSystemId, diPort;
  bool bRes, bConnected;
  dyn_dyn_string ddsTemp;
  
//  DebugTN("unicosHMI_registerTreeDeviceOverview");
/*-------------------------------------------------------------------------------------------*/
// get the list of the systems connected to the system  
/*-------------------------------------------------------------------------------------------*/
  unDistributedControl_getAllDeviceConfig(dsSystemNames, diSystemId, dsHost, diPort);
  len = dynlen(dsSystemNames);
  synchronized(g_dsTreeApplication) {
    for(i=1;i<=len;i++) {
  //DebugTN("start unicosHMI_registerTreeDeviceOverview", dsSystemNames[i]);
      if(strpos(dsSystemNames[i], ":") < 0)
        dsSystemNames[i] = dsSystemNames[i]+":";
      g_dsTreeDeviceOverviewRegisteredSystem[i] = dsSystemNames[i];
      g_m_bTreeDeviceOverviewRegisteredSystem[g_dsTreeDeviceOverviewRegisteredSystem[i]] = false;
      g_m_bTreeDeviceOverviewSystemConnected[g_dsTreeDeviceOverviewRegisteredSystem[i]] = false;
      g_m_ddsTreeDeviceOverviewDeviceList[g_dsTreeDeviceOverviewRegisteredSystem[i]] = ddsTemp;
      g_m_bTreeDeviceOverviewRegisteredSystemInitialized[dsSystemNames[i]] = false;
      g_m_bTreeDeviceOverview_gettingDeviceRunning[dsSystemNames[i]] = false;
      g_m_bTreeDeviceOverview_deviceListAvailable[dsSystemNames[i]] = false;

      g_m_sTreePVSSSystem[g_dsTreeDeviceOverviewRegisteredSystem[i]] = "";
      g_m_dsTreeFrontEnd[g_dsTreeDeviceOverviewRegisteredSystem[i]] = makeDynString();
      g_m_dsTreeFrontEndDp[g_dsTreeDeviceOverviewRegisteredSystem[i]] = makeDynString();
      g_m_dsTreeFrontEndType[g_dsTreeDeviceOverviewRegisteredSystem[i]] = makeDynString();
      g_m_dsTreeApplication[g_dsTreeDeviceOverviewRegisteredSystem[i]] = makeDynString();
      g_m_dsTreeFrontEndApplication[g_dsTreeDeviceOverviewRegisteredSystem[i]] = ddsTemp;
      g_m_dsTreeSubsystem1[g_dsTreeDeviceOverviewRegisteredSystem[i]] = makeDynString();
      g_m_dsTreeSubsystem2[g_dsTreeDeviceOverviewRegisteredSystem[i]] = makeDynString();
    }
  }
  for(i=1;i<=len;i++) {
    unDistributedControl_register("_unicosHMI_TreeDeviceOverview_register", bRes, bConnected, dsSystemNames[i], exInfo);
//DebugTN(bRes, bConnected, dsSystemNames[i]);
  }
  return true;
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unicosHMI_waitTreeDeviceOverviewRegistration
/**
Purpose:
End initialisation function. Put here any code to be executed once at startup after the initialisation. 
This function is executed by evalScript

  @param return value: bool, output, true=OK/false=BAD

Usage: Public

PVSS manager usage: UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
  . function executed by evalScript->no dpConnect, dpQueryConnect, startThread, etc.
  . PVSS version: 3.6 
  . operating system: WXP and Linux.
  . distributed system: yes.
*/
bool unicosHMI_waitTreeDeviceOverviewRegistration()
{
  int i, len;
  dyn_string dsSystemNames, dsHost, exInfo;
  dyn_int diSystemId, diPort;
  
//  DebugTN("unicosHMI_waitTreeDeviceOverviewRegistration");
/*-------------------------------------------------------------------------------------------*/
// get the list of the systems connected to the system  
/*-------------------------------------------------------------------------------------------*/
  unDistributedControl_getAllDeviceConfig(dsSystemNames, diSystemId, dsHost, diPort);
  len = dynlen(dsSystemNames);

// wait until the initialization is done
  for(i=1;i<=len;i++) {
   if(strpos(dsSystemNames[i], ":") < 0)
     dsSystemNames[i] = dsSystemNames[i]+":";

   unGraphicalFrame_setLoadingText("Waiting ack ("+i+"/"+len+") from "+dsSystemNames[i]);

   while(!mappingHasKey(g_m_bTreeDeviceOverviewRegisteredSystemInitialized, dsSystemNames[i]))
     delay(1);
   while(!g_m_bTreeDeviceOverviewRegisteredSystemInitialized[dsSystemNames[i]])
     delay(1);
   g_m_bTreeDeviceOverview_gettingDeviceRunning[dsSystemNames[i]] = false;
   unGraphicalFrame_setLoadingText("Connection ("+i+"/"+len+") to "+dsSystemNames[i]+" OK");
//DebugTN("reg done", dsSystemNames[i], g_m_bTreeDeviceOverviewRegisteredSystemInitialized[dsSystemNames[i]], "done");
   unGraphicalFrame_triggerDisplay(0, dsSystemNames[i]);
  }

  unGraphicalFrame_setLoadingText("Initialisation completed");
//  DebugTN("!!end unicosHMI_waitTreeDeviceOverviewRegistration");
  return true;
}

//--------------------------------------------------------------------------------------------------------------------------------

// _unicosHMI_TreeDeviceOverview_register
/**
Purpose:
Distributed control register callback function

  @param sDpe: string, input:  the distributed control DPE of a PVSS system.
  @param bSystemConnected: bool, input: state of the system

Usage: Public

PVSS manager usage: UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
  . PVSS version: 3.6 
  . operating system: WXP and Linux.
  . distributed system: yes.
*/
_unicosHMI_TreeDeviceOverview_register(string sDpe, bool bSystemConnected)
{
  string sSystemName;
  string sDp = dpSubStr(sDpe, DPSUB_DP);
  bool bRemote, localConnected;
  dyn_dyn_string ddsTemp;
  int i, len;
  
  synchronized(g_dsTreeApplication) {
// get the system name and check if it is local or remote
    sSystemName = substr(sDp, strlen(c_unDistributedControl_dpName), strlen(sDp))+":";
//  DebugTN("start _unicosHMI_TreeDeviceOverview_register", sSystemName, bSystemConnected);
    unDistributedControl_isRemote(bRemote, sSystemName);
  //DebugN("unTreeDeviceOverview_register", g_m_bTreeDeviceOverviewRegisteredSystem, sDp, bSystemConnected, bRemote, sSystemName, mappingHasKey(g_m_bTreeDeviceOverviewRegisteredSystem, sSystemName));
    if(mappingHasKey(g_m_bTreeDeviceOverviewRegisteredSystem, sSystemName)) {
    //DebugN("has key");
       if(bRemote)
         localConnected = bSystemConnected;
       else
         localConnected = true;
       if(localConnected) { //the system is connected
        // if the system is not already connected add it: trigger the display cb
        // else do nothing
         if(!g_m_bTreeDeviceOverviewRegisteredSystem[sSystemName]) {
            g_m_bTreeDeviceOverviewRegisteredSystem[sSystemName] = true;
            g_m_bTreeDeviceOverviewSystemConnected[sSystemName] = true;
        //    g_m_ddsTreeDeviceOverviewDeviceList[sSystemName] = ddsTemp;
        //DebugTN("set connc", sSystemName);

          if(!g_m_bTreeDeviceOverviewRegisteredSystemInitialized[sSystemName]) { // startup, start thread
             g_m_bTreeDeviceOverview_gettingDeviceRunning[sSystemName] = true;
      //     startThread("unTreeDeviceOverview_loadAll", sSystemName, g_dsDeviceType);
             unTreeDeviceOverview_loadAll(sSystemName, g_dsDeviceType);
          }
         }
       }
       else { // the system is disconnected
        // if the system is connected delete it: trigger the display cb
        // else do nothing
         if(g_m_bTreeDeviceOverviewRegisteredSystem[sSystemName]) {
          g_m_bTreeDeviceOverviewRegisteredSystem[sSystemName] = false;
          g_m_bTreeDeviceOverviewSystemConnected[sSystemName] = false;
      //    g_m_ddsTreeDeviceOverviewDeviceList[sSystemName] = ddsTemp;

          g_m_sTreePVSSSystem[sSystemName] = "";
          g_m_dsTreeFrontEnd[sSystemName] = makeDynString();
          g_m_dsTreeFrontEndDp[sSystemName] = makeDynString();
          g_m_dsTreeFrontEndType[sSystemName] = makeDynString();
          g_m_dsTreeApplication[sSystemName] = makeDynString();
          g_m_dsTreeFrontEndApplication[sSystemName] = ddsTemp;
          g_m_dsTreeSubsystem1[sSystemName] = makeDynString();
          g_m_dsTreeSubsystem2[sSystemName] = makeDynString();

          _unTreeDeviceOverview_buildSubsystemList();

      //DebugN("set disc", sSystemName);
          if(g_m_bTreeDeviceOverviewRegisteredSystemInitialized[sSystemName]) // startup, start thread
           unGraphicalFrame_triggerDisplay(DEVUN_DELETE_COMMAND, sSystemName);
         }
         g_m_bTreeDeviceOverviewRegisteredSystemInitialized[sSystemName] = true;
       }
     }
  }
//DebugTN("end _unicosHMI_TreeDeviceOverview_register", sSystemName);
}

//---------------------------------------------------------------------------------------------------------------------------------------

//@}
